<?php
/**
 * @name Model_adm
 * @description Classe modèle de la partie d'administration d'Esia
 *		Toutes les méthodes sont déclarée en 'static' ainsi on garde les Objets PDO pour les programmeurs extérieurs et pour faire des modifications, ils ne sont pas /
 *		obligé de connaitre la hierachie de classe d'ESIA
 * @author Nicolas Biersart <n.biersart@esia-network.com>
 * @contributor
 * @copyright Nicolas Biersart 2012
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date 10/04/2013
 */
include_once DEVICE_CLASS;
include_once GESA_CLASS;
include_once MVCMODEL;
//include_once $path.'/plugin/gesa/libs/gesa.class.php';
class Model_adm extends Model_esia
{
	/**
	 * retourne un tableau associatif contenant les informations sur les droits d'accès.
	 * @name getAccess
	 * @param PDO $dbconnection
	 * @param integer $id (optionnelle) id du droit d'utilisation
	 * @retval un tableau associatif organiser comme suit:
	 * 		$arr[<privilege_id>][<privilege_id>]=identifiant du privilège
	 *		$arr[<privilege_id>][<privilege_name>]=nom du privilège
	 * 		$arr[<privilege_id>][<privilege_site>]=si il se rapporte à un droit sur un noeud ou un droit sur le site
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getAccess($dbconnection,$id = null)
	{
		if(isset($id))
		{
			$query =  $dbconnection->prepare("SELECT privilege_id,privilege_name,privilege_site,privilege_description FROM esia_privileges WHERE privilege_id =?;");
			parent::executeStatement($query,array($id));
		}
		else
		{
			$query =  $dbconnection->prepare("SELECT privilege_id,privilege_name,privilege_site,privilege_description FROM esia_privileges;");
			parent::executeStatement($query);
		}
		$list_access=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$list_access[$row->privilege_id] = array('id'   => $row->privilege_id,
					'name' => $row->privilege_name,
					'site' => $row->privilege_site,
			    'description' => utf8_encode($row->privilege_description));
		}
		return $list_access;
	}

	/**
	 * retourne un tableau associatif contenant les dernières lignes des fichiers de logs
	 * @name getDaemonLog
	 * @param PDO $dbconnection
	 * @param integer $type si Model_esia::LOG_DAEMON_CHECK log du démon de check
	 *	      				si model_esia::LOG_DAEMON_TRAP log du démon de trap
	 * @retval $arr[]=array('date'=>$in[0],'state'=>$in[1],'message'=>$in[2]);
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getDaemonLog($dbconnection,$type=Model_esia::LOG_DAEMON_CHECK)
	{
		if($type == model_adm::LOG_DAEMON_TRAP)
		{
			$query = $dbconnection->prepare("SELECT config_key, config_value
					FROM esia_configuration
					WHERE config_key = 'SYS_ESIATRAPD_LOG';");
		}
		else
		{
			$query = $dbconnection->prepare("SELECT config_key, config_value
					FROM esia_configuration
					WHERE config_key = 'SYS_ESIADAEMON_LOG';");
		}
		$query = parent::executeStatement($query);
		$row=$query->fetch(PDO::FETCH_OBJ);
		$lines = Model_adm::tail($row->config_value, 10);
		$arr= array();
		foreach($lines as $line)
		{
			$in = preg_split('/#/',$line);
			if($in[0]!=$line)
			{
				$arr[]=array('date'=>$in[0],'state'=>$in[1],'message'=>$in[2]);
			}
		}
		return $arr;
	}

	/**
	 * retourne un tableau associatif contenant les dernières lignes des fichiers de logs
	 * @name countServiceByPlugin
	 * @param PDO $dbconnection
	 * @param integer $id id du plugin
	 * @retval le nom de service exécutant se plugin
	 * @exception EsiaDbException si une erreur survient
	 */
	static function countServiceByPlugin($dbconnection,$id)
	{
		$query= $dbconnection->prepare("SELECT count(*) as nbr
				FROM esia_services
				WHERE service_plugin_id=?;");
		$query = parent::executeStatement($query,array($id));
		$row=$query->fetch(PDO::FETCH_OBJ);
		return $row->nbr;
	}

	/**
	 * équivalent de la fonction tail en shell retourne les X dernière ligne d'un fichier
	 * @name tail
	 * @param string $file chemin d'accès au fichier de log
	 * @param integer $lines nombre de lignes à retourner
	 * @retval un tableau contenant les X dernières lignes du fichier
	 * @exception /
	 */
	private static function tail($file, $lines)
	{
		//global $fsize;
		$handle = @fopen($file, "r");
		if($handle!==FALSE)
		{
			$linecounter = $lines;
			$pos = -2;
			$beginning = false;
			$text = array();
			while ($linecounter > 0)
			{
				$t = " ";
				$buff="";
				while ($t != "\n") {
					if(fseek($handle, $pos, SEEK_END) == -1) {
						$beginning = true;
						break;
					}
					$t = fgetc($handle);
					$buff.=$t;
					$pos --;
				}
				$linecounter --;
				$text[] = strrev($buff);
				if ($beginning) break;
			}
			fclose ($handle);
			return array_reverse($text);
		}
		else
		{
			return array();
		}
	}

	/**
	 * retourne un tableau associatif contenant les 5 derniers plugins exécutés par esiaDaemon
	 * @name getLastExecCheckPlugin
	 * @param PDO $dbconnection
	 * @retval $arr[]= array('device'=>$row->device_name,
 	 *						 'name'=>$row->service_name,
	 *						 'page_control'=>$page_control,
	 *						 'state' =>$row->service_previous_state,
	 *						 'message'=>$row->service_last_stdout,
	 *						 'date'=>$row->timeexec);
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getLastExecCheckPlugin($dbconnection)
	{
		$query = $dbconnection->prepare("SELECT device_name,device_type,service_previous_state, to_char(service_last_exec_time,'DD/MM/YYYY HH24:MI:SS') as timeexec, service_last_stdout, service_name
				FROM esia_services
				JOIN esia_device ON service_device_id = device_id
				ORDER BY service_last_exec_time
				LIMIT 5;");
		$query = parent::executeStatement($query);
		$arr= array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$page_control = Model_esia::getPageControl($dbconnection, $row->device_type);
			if($page_control!=null)
			{
				$page_control = preg_replace('#:name#',$row->device_name,$page_control);
			}
			$arr[]= array('device'=>$row->device_name,
					'name'=>$row->service_name,
					'page_control'=>$page_control,
					'state' =>$row->service_previous_state,
					'message'=>$row->service_last_stdout,
					'date'=>$row->timeexec
			);
		}
		return $arr;

	}

	/**
	 * récupère les groupes d'utilisateurs
	 * @name getGroup
	 * @param PDO $dbconnection
	 * @param integer|string $id (optionnel) id ou nom du group a récupérer
	 * @retval un tableau contenant
	 *	array[<id du group>]['id']	id du groupe
	 *	array[<id du group>]['name'] nom du groupe
	 *	array[<id du group>]['description'] description du groupe
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getGroup($dbconnection,$id=null)
	{
		if(isset($id)&&is_numeric($id))
		{
			$query = $dbconnection->prepare("SELECT group_id,group_name,group_description FROM esia_groups WHERE group_id=?");
			$query = parent::executeStatement($query,array($id));
		}
		elseif(isset($id))
		{
			$query = $dbconnection->prepare("SELECT group_id,group_name,group_description FROM esia_groups WHERE group_name LIKE ?");
			$query = parent::executeStatement($query,array('%'.$id.'%'));
		}
		else
		{
			$query = $dbconnection->prepare("SELECT group_id,group_name,group_description FROM esia_groups;");
			$query = parent::executeStatement($query);
		}
		$list_grp=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$list_grp[$row->group_id]=array('id'=>$row->group_id,
					'name'=> $row->group_name,
					'description'=>$row->group_description);
		}
		return $list_grp;

	}

	/**
	 * récupère les privilèges alloués au groupe donné
	 * @name getGroupPrivilege
	 * @param PDO $dbconnection
	 * @param integer|string $id id ou nom du groupe
	 * @retval un tableau contenant
	 *	array[<privilege_id>]['id']	id du droit
	 *	array[<privilege_id>]['name'] nom du droit
	 *	array[<privilege_id>]['description'] description du droit
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getGroupPrivilege($dbconnection,$id=null)
	{
		$sql = "SELECT privilege_id,privilege_name,privilege_description
		FROM esia_privileges
		LEFT JOIN esia_privilege_grantedto ON privilege_id = grant_privilege_id
		LEFT JOIN esia_groups ON group_id = grant_group_id ";
		if(isset($id)&&is_numeric($id))
		{
		    $sql = $sql.'WHERE group_id = ?';
		}
		else
		{
		    $sql = $sql.'WHERE group_name = ?';
		}
		$sql = $sql.' ORDER BY privilege_name';
		$query = $dbconnection->prepare($sql);
		$query = parent::executeStatement($query,array($id));
		$list_privi=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$list_privi[$row->privilege_id]=array('id'=>$row->privilege_id,
					'name'=> $row->privilege_name,
					'description'=>$row->privilege_description);
		}
		return $list_privi;

	}

	/**
	 * récupère les privilèges non alloués au groupe donné
	 * @name getGroupNotPrivilege
	 * @param PDO $dbconnection
	 * @param string|integer $id (optionnel) id ou nom du groupe
	 * @retval un tableau contenant
	 *	array[<privilege_id>]['id']	id du droit
	 *	array[<privilege_id>]['name'] nom du droit
	 *	array[<privilege_id>]['description'] description du droit
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getGroupNotPrivilege($dbconnection,$id=null)
	{
		$sql = "SELECT privilege_id,privilege_name,privilege_description
		FROM esia_privileges
		EXCEPT
		SELECT privilege_id,privilege_name,privilege_description
		FROM esia_privileges
		LEFT JOIN esia_privilege_grantedto ON privilege_id = grant_privilege_id
		LEFT JOIN esia_groups ON group_id = grant_group_id ";
		if(isset($id)&&is_numeric($id))
		{
		    $sql = $sql.'WHERE group_id = ?';
		}
		else
		{
		    $sql = $sql.'WHERE group_name = ?';
		}
		$sql = $sql.'ORDER BY privilege_name';
		$query = $dbconnection->prepare($sql);
		$query = parent::executeStatement($query,array($id));
		$list_privi=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$list_privi[$row->privilege_id]=array('id'=>$row->privilege_id,
					'name'=> $row->privilege_name,
					'description'=>$row->privilege_description);
		}
		return $list_privi;

	}
	/**
	 * Retourne les parents possibles
	 * @param PDO $dbconnection
	 * @param integer $id du groupe de device
	 * @retval array contenant les parents possible
	 * @exception 	EsiaDbException si une erreur survient
	 */
	static function getGroupDeviceParent($dbconnection,$id)
	{
		$query = $dbconnection->prepare("SELECT device_group_id,device_group_name,device_group_description,device_group_parent
		      FROM esia_device_group
			  WHERE device_group_id != :id
			  AND device_group_id NOT IN (SELECT group_id FROM getallchilddevicegroup(:id))
			  ORDER BY device_group_order;");
		$query = parent::executeStatement($query, array(':id'=> $id));
		$list_node=array();

		$req = $dbconnection->prepare("WITH RECURSIVE q AS (SELECT device_group_id, device_group_name,device_group_parent,device_group_order
	                      									FROM esia_device_group
	 														WHERE device_group_id = :id
	                    									UNION ALL
	                      									SELECT m.device_group_id, m.device_group_name,m.device_group_parent,m.device_group_order
	                        								FROM esia_device_group m
	                        								JOIN q ON q.device_group_parent = m.device_group_id)
	 									SELECT * FROM q ");

		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$req = parent::executeStatement($req, array(':id'=> $row->device_group_id));
			$tmpname="";
			while($val=$req->fetch(PDO::FETCH_OBJ))
			{
				if($tmpname=="")
				{
					$tmpname = $val->device_group_name;
				}
				else
				{
					$tmpname = $val->device_group_name.">".$tmpname;
				}
			}
			$row->device_group_name=$tmpname;
			$list_node[]=array('id'=>$row->device_group_id ,'name'=>$row->device_group_name,'description'=>$row->device_group_description,'parent'=>$row->device_group_parent);
		}
		return $list_node;
	}

	/**
	 * récupère les noeuds auquelle le groupe a le droit d'accès
	 * @name getGroupAccessNode
	 * @param PDO $dbconnection
	 * @param string|integer $id id ou nom du groupe
	 * @retval un tableau contenant
	 *	array[<node id>]['id']	id du noeud
	 *	array[<node id>]['name'] nom du noeud
	 *	array[<node id>]['group'] nom du groupe de noeud
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getGroupAccessNode($dbconnection,$id=null,$noaccess=false)
	{
	    $sql = "SELECT device_id,device_name,g2.device_group_name
		FROM esia_device
		LEFT JOIN esia_device_group_in ON device_id = group_in_device_id
		LEFT JOIN esia_device_group  as g2 ON device_group_id= group_in_device_group_id";
	    if($noaccess==true)
	    {
	        $sql = $sql.' EXCEPT
	        SELECT device_id,device_name,g2.device_group_name
	        FROM esia_device
	        LEFT JOIN esia_device_group_in ON device_id = group_in_device_id
	        LEFT JOIN esia_device_group  as g2 ON device_group_id= group_in_device_group_id
	        JOIN esia_group_device_access ON access_device_id= device_id
	        JOIN esia_groups ON group_id = access_group_id ';
	    }
	    else
	    {
    		$sql = $sql.' JOIN esia_group_device_access ON access_device_id= device_id
    		JOIN esia_groups ON group_id = access_group_id ';
	    }
		if(isset($id)&&is_numeric($id))
		{
		    $sql = $sql.'WHERE group_id = ?';
		}
		else
		{
			$sql = $sql.'WHERE group_name= ?';
		}
		$sql.=' ORDER BY device_name';
		$query = $dbconnection->prepare($sql);
		$query = parent::executeStatement($query,array($id));
		$list_node=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
		    $groupname = str_replace(array('(',')','.','[',']',':',';','%','|'),'',$row->device_group_name);
		    $list_node[$groupname][]=array('id'=>$row->device_id,
		        'name'=> $row->device_name);
		}
		return $list_node;
	}

	/**
	 * récupère les noeuds auquelle le groupe n'a pas le droit d'accès
	 * @name getGroupNoAccessNode
	 * @param PDO $dbconnection
	 * @param integer|string $id id ou nom du groupe
	 * @retval un tableau contenant
	 *	array[<node group>]['id']	id du noeud
	 *	array[<node group>]['name'] nom du noeud
	 *	array[<node group>]['group'] nom du groupe de noeud
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getGroupNoAccessNode($dbconnection,$id=null)
	{
	    return Model_adm::getGroupAccessNode($dbconnection,$id,true);
	}

	/**
	 * retourne les patterns de service si rien n'est spécifié => tout les patterns
	 *										si id est spécifié => le pattern demandé
	 *										si node est spécifié => les patterns associés au noeud
	 * @param PDO $dbconnection
	 * @param integer $id (optionnelle) identifiant du pattern
	 * @param integer $node (optionnelle) identifant d'un noeud
	 * @retval un tableau contenant
	 *	array[<pattern_id>]['id']	identifiant du pattern
	 *	array[<pattern_id>]['name'] nom du pattern
 	 *	array[<pattern_id>]['description'] sa description
	 *	array[<pattern_id>]['plugins']	tableau des plugins composant le pattern
 	 *	array[<pattern_id>]['plugins']['id'] id du plugin au sein du pattern (esia_service_pattern_included)
	 *	array[<pattern_id>]['plugins']['name'] nom du service
 	 *	array[<pattern_id>]['plugins']['parameter'] ses paramètres
	 *	array[<pattern_id>]['plugins']['interval'] interval d'exécution
 	 *	array[<pattern_id>]['plugins']['plugin_type'] type (management,default,trap,etc)
	 *	array[<pattern_id>]['plugins']['plugin_id'] id du plugin
 	 *	array[<pattern_id>]['plugins']['alerting'] id du programme d'alerte
	 *	array[<pattern_id>]['plugins']['alerting_parameter'] paramètre du programme d'alerte
 	 *	array[<pattern_id>]['plugins']['unknown'] si il est exécuté en cas d'erreur inconnue
	 *	array[<pattern_id>]['plugins']['warning'] si il est exécuté en cas d'erreur warning
     *	array[<pattern_id>]['plugins']['critical'] si il est exécuté en cas d'erreur critique
	 *	array[<pattern_id>]['plugins']['priority'] priorité du service
	 * @throws EsiaDbException si une erreur survient
	 */
	static function getPatternService($dbconnection,$id=null,$node=null)
	{
		$param=array();
		if(isset($node)&&is_numeric($node))
		{
			$sql = 'SELECT pattern_id,pattern_name,pattern_description,included_id,
							included_name,included_parameter,included_interval,plugin_id,plugin_type,plugin_name,
							included_program_id,included_program_parameter,included_unknown,included_warning,
							included_critical,included_priority,plugin_path
					FROM esia_service_patterns
					JOIN esia_service_pattern_included ON pattern_id = included_pattern_id
					JOIN esia_plugins ON plugin_id = included_plugin_id
					JOIN esia_services ON service_included_id = included_id
					WHERE service_device_id = ? AND service_delete=FALSE ';
			$param[]=$node;
		}
		else
		{
			$sql = 'SELECT pattern_id,pattern_name,pattern_description,included_id,included_name,included_parameter,included_interval,plugin_id,plugin_type,plugin_name,
						included_program_id,included_program_parameter,included_unknown,
						included_warning,included_critical,included_priority,included_ok,plugin_path
					FROM esia_service_patterns
					LEFT JOIN esia_service_pattern_included ON pattern_id = included_pattern_id
					LEFT JOIN esia_plugins ON plugin_id = included_plugin_id';
			if(isset($id))
			{
				if(is_numeric($id))
				{
					$sql.=' WHERE pattern_id = ?;';
				}
				else
				{
					$sql.=' WHERE pattern_name= ?;';
				}
				$param[]=$id;
			}
		}
		$query = $dbconnection->prepare($sql);
		$query = parent::executeStatement($query,$param);
		$list=array();
		$plugins = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$list[$row->pattern_id]=array('id'=>$row->pattern_id,'name'=>$row->pattern_name,'description'=>$row->pattern_description);
			if(!empty($row->included_id))
			{
				$plugins[]=array('id'=>$row->included_id,
						'name'=>$row->included_name,
						'parameter'=>$row->included_parameter,
						'interval'=>$row->included_interval,
						'plugin_type'=>$row->plugin_type,
						'plugin_id'=>$row->plugin_id,
						'plugin_name'=>$row->plugin_name,
						'alerting'=>$row->included_program_id,
						'alerting_parameter'=> $row->included_program_parameter,
						'unknown'=> $row->included_unknown,
						'warning'=> $row->included_warning,
						'critical'=> $row->included_critical,
						'ok'=> $row->included_ok,
						'priority'=>$row->included_priority,
						'path'=>$row->plugin_path
						);
			}
			$list[$row->pattern_id]['plugins'] = $plugins;

		}
		return $list;

	}

	/**
	 * retourne les noeud associé à un pattern donné
	 * @name getPatternService
	 * @param PDO $dbconnection
	 * @param integer $id identifiant du pattern
	 * @retval un tableau contenant
	 * 	array[]['id'] id du noeud
	 * 	array[]['name'] nom du noeud
	 * 	array[]['description'] description du noeud
	 * 	array[]['type'] type du noeud
	 * 	array[]['group'] group du noeud
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getPatternServiceNodeAssign($dbconnection,$id)
	{
		$list=array();
		if(isset($id)&&is_numeric($id))
		{
			$query = $dbconnection->prepare("SELECT DISTINCT device_id,device_name,device_description,device_group_name,device_type
											FROM esia_device
											JOIN esia_group_device_access ON access_device_id = device_id
											JOIN esia_services ON service_device_id = device_id
											JOIN esia_service_pattern_included ON service_included_id= included_id
											WHERE device_delete = 'false'
											AND included_pattern_id = ?");
			$query = parent::executeStatement($query,array($id));
			while($row=$query->fetch(PDO::FETCH_OBJ))
			{
				$list[]=array('id'=>$row->device_id,
							  'name'=>$row->device_name,
							  'description'=>$row->device_description,
							  'type'=>$row->device_type,
							  'group'=>$row->device_group_name);
			}
		}
		return $list;
	}

	/**
	 * Compte pour chacun des included_id du pattern le nombre de services associés qui n'ont pas la même conf
	 * @name getPatternIncludedNotSameThanService
	 * @param PDO $dbconnection
	 * @param integer $pattern_id identifiant du pattern
	 * @retval un tableau contenant
	 * 	array[]['id'] id de l'included
	 * 	array[]['name'] nom de l'included
	 * 	array[]['plugin'] plugin id de l'included
	 * 	array[]['parameter'] parameters de l'included
	 * 	array[]['count'] le nombre de services associés dont les paramètres et alertes sont différentes
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getPatternIncludedNotSameThanService($dbconnection,$pattern_id)
	{
		$sql = 'SELECT included_id,included_name,included_plugin_id,included_parameter,count(service_id) AS count';
		$sql.= ' FROM esia_service_pattern_included';
		$sql.= ' INNER JOIN esia_services';
		$sql.= ' ON(service_included_id = included_id)';
		$sql.= ' LEFT JOIN esia_service_response_program';
		$sql.= ' ON(service_id = response_service_id)';
		$sql.= ' WHERE included_pattern_id = :pattern_id';
		$sql.= " AND (
			service_parameter != included_parameter
			OR service_interval_time != included_interval
			OR COALESCE(response_program_id,0) != included_program_id
			OR COALESCE(response_parameter,'') != included_program_parameter
			OR COALESCE(response_ok,false) != included_ok
			OR COALESCE(response_warning,false) != included_warning
			OR COALESCE(response_critical,false) != included_critical
			OR COALESCE(response_unknown,false) != included_unknown
		)";
		$sql.= ' GROUP BY included_id';
		$query = $dbconnection->prepare($sql);
		$query = parent::executeStatement($query,array($pattern_id));

		$list = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$list[]=array('id'=>$row->included_id,
						  'name'=>$row->included_name,
						  'plugin'=>$row->included_plugin_id,
						  'parameter'=>$row->included_parameter,
						  'count'=>$row->count);
		}
		return $list;
	}


	/**
	 * Retourne les included qui ne snt pas encore ajoutés dans des noeuds
	 * @name getPatternNewIncluded
	 * @param PDO $dbconnection
	 * @param integer $pattern_id identifiant du pattern
	 * @retval un tableau contenant
	 * 	array[]['id'] id de l'included
	 * 	array[]['name'] nom de l'included
	 * 	array[]['plugin'] plugin id de l'included
	 * 	array[]['parameter'] parameters de l'included
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getPatternNewIncluded($dbconnection,$pattern_id)
	{
		$sql = 'SELECT included_id,included_name,included_plugin_id,included_parameter';
		$sql.= ' FROM esia_service_pattern_included';
		$sql.= ' WHERE included_pattern_id = :pattern_id';
		$sql.= ' AND included_id NOT IN (';
		$sql.= ' SELECT service_included_id FROM esia_services';
		$sql.= ' WHERE service_delete = FALSE AND service_included_id IS NOT NULL)';

		$query = $dbconnection->prepare($sql);
		$query = parent::executeStatement($query,array($pattern_id));

		$list = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$list[]=array('id'=>$row->included_id,
						  'name'=>$row->included_name,
						  'plugin'=>$row->included_plugin_id,
						  'parameter'=>$row->included_parameter);
		}
		return $list;
	}

	/**
	 * Retourne des stats sur le status des noeuds par rapport à un pattern
	 * @name getPatternServiceNodeStats
	 * @param PDO $dbconnection
	 * @param array $node_list identifiants d'un ensemble de noeuds sous forme d'id liste
	 * @param integer $pattern_id identifiant du pattern
	 * @retval un tableau contenant une liste de noeuds sous la forme
	 * 	array[node_id]['SERVICES']['INCLUDED'] => les services inclus dans ce pattern pour le noeud
	 * 	array[node_id]['SERVICES']['MODIFIED'] => les services inclus mais différents
	 * 	array[node_id]['SERVICES']['DUPLICATED'] => les services en double (pattern ajouté plusieurs fois au noeud)
	 * 	array[node_id]['SERVICES']['NO_PATTERN'] => les services qui n'ont pas de pattern
	 * 	array[node_id]['SERVICES']['ANOTHER_PATTERN'] => les services présent dans d'autres patterns
	 * 	array[node_id]['NOT_INCLUDED'] => les included qui n'ont pas de services associés dans le noeud
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getPatternServiceNodeStats($dbconnection,$node_list,$pattern_id)
	{
		$node_sql = 'SELECT *';
		$node_sql.= ' FROM esia_device';
		$node_sql.= ' INNER JOIN esia_services ON (device_id = service_device_id)';
		$node_sql.= ' INNER JOIN esia_plugins ON  (plugin_id = service_plugin_id)';
		$node_sql.= ' LEFT OUTER JOIN esia_service_pattern_included ON (service_included_id = included_id )';
		$node_sql.= ' LEFT OUTER JOIN esia_service_patterns ON (included_pattern_id = pattern_id)';
		$node_sql.= ' LEFT OUTER JOIN esia_service_response_program ON (service_id = response_service_id)';
		$node_sql.= ' WHERE device_id = :device_id AND service_delete = FALSE';
		$node_sql.= ' ORDER BY service_id ASC';

		$included_sql = 'SELECT included_id,included_name';
		$included_sql.= ' FROM esia_service_pattern_included';
		$included_sql.= ' WHERE included_pattern_id = :pattern_id';
		$included_sql.= ' AND included_id NOT IN(';
		$included_sql.= ' SELECT service_included_id FROM esia_services';
		$included_sql.= ' WHERE service_device_id = :device_id AND service_delete = FALSE AND service_included_id IS NOT NULL)';

		$node_query = $dbconnection->prepare($node_sql);
		$included_query = $dbconnection->prepare($included_sql);

		$ret = array();
		foreach($node_list as $node)
		{
			$query = self::executeStatement($node_query,array(':device_id'=>$node['id']));
			$services['INCLUDED'] 			= array();
			$services['MODIFIED']			= array();
			$services['DUPLICATED']			= array();
			$services['NO_PATTERN'] 		= array();
			$services['ANOTHER_PATTERN'] 	= array();
			$duplicate_check = array();

			$i=0;
			while($row=$query->fetch(PDO::FETCH_ASSOC))
			{
				$i++;
				if($row['included_id'] === NULL )
				{
					$services['NO_PATTERN'][] = array(
						'service_id' => $row['service_id'],
						'service_name' => $row['service_name'],
						'plugin_id' => $row['plugin_id'],
						'plugin_name' => $row['plugin_name'],
						'included_id' => $row['included_id'],
						'included_name' => $row['included_name']
					);
				}
				else if( $row['pattern_id'] != $pattern_id )
				{
					$services['ANOTHER_PATTERN'][] = array(
						'service_id' => $row['service_id'],
						'service_name' => $row['service_name'],
						'plugin_id' => $row['plugin_id'],
						'plugin_name' => $row['plugin_name']
					);
				}
				else if( $row['pattern_id'] == $pattern_id )
				{
					$included = array(
						'service_id' => $row['service_id'],
						'service_name' => $row['service_name'],
						'plugin_id' => $row['plugin_id'],
						'plugin_name' => $row['plugin_name'],
						'included_id' => $row['included_id'],
						'included_name' => $row['included_name']
					);
					if( !isset($duplicate_check[$row['included_id']]) )
					{
						if(
							$row['service_parameter'] != $row['included_parameter']
							|| $row['service_interval_time'] != $row['included_interval']
							|| $row['response_program_id'] != $row['included_program_id']
							|| $row['response_parameter'] != $row['included_program_parameter']
							|| $row['response_ok'] != $row['included_ok']
							|| $row['response_warning'] != $row['included_warning']
							|| $row['response_critical'] != $row['included_critical']
							|| $row['response_unknown'] != $row['included_unknown']
						)
						{
							$services['MODIFIED'][] = $included;
						}
						else
						{
							$services['INCLUDED'][] = $included;
						}
					}
					else
					{
						$services['DUPLICATED'][] = $included;
					}
					$duplicate_check[$row['included_id']] = $row;
				}
			}

			$not_included = array();
			$query = self::executeStatement($included_query,array(':device_id'=>$node['id'],':pattern_id'=>$pattern_id));
			while($row=$query->fetch(PDO::FETCH_ASSOC))
			{
				$not_included[] = $row;
			}

			$ret[$node['id']] = array('SERVICES'=>$services,'NOT_INCLUDED'=>$not_included);
		}
		return $ret;
	}

	/**
	 * Insère l'objet device dans la base de donnée.
	 * @name addNode
	 * @param PDO $dbconnection
	 * @param Device $dev L'objet Device à insérer dans la base de donnée
	 * @retval l'id du nouveau device
	 * @exception EsiaDbException si une erreur survient
	 */
	public static function addNode($dbconnection, Device $dev )
	{
		$query = $dbconnection->prepare("INSERT INTO esia_device (
				device_id, device_name, device_type,device_ip_address,device_protocol_connection, device_username, device_passwd,
				device_snmp_version,device_snmp_timer,device_snmp_community,device_snmp3_user,device_snmp3_sec_lvl,
				device_snmp3_auth_protocole,device_snmp3_auth_key,device_snmp3_priv_protocol,device_snmp3_priv_passphrase,
				device_group_name,device_description,device_snmp_port )
				VALUES
				(DEFAULT,:name,:type,:ip,:protocol,:login,:pwd,:snmp_ver,:snmp_timer,:community,:snmp3_user,:snmp3_seclvl,
				:snmp3_authprot,:snmp3_authkey,:snmp3_privprot,:snmp3_privpass,:grp,:description,:snmp_port) RETURNING device_id;");
		//$query->bindParam(":pwd", $pass, PDO::PARAM_LOB);
		$arr = Model_adm::nodeObjectToArray($dev);
		$query = parent::executeStatement($query,$arr);
		$row=$query->fetch(PDO::FETCH_OBJ);
		$query = $dbconnection->prepare("INSERT INTO esia_device_group_in(group_in_device_id, group_in_device_group_id)VALUES (:id, :group);");
		$grps =  $dev->getGroupList();
		foreach($grps as $grp)
		{
			$query = parent::executeStatement($query,array(':id' => $row->device_id,':group'=>$grp));
		}
		global $lang;
		model_adm::addGdprRegister($dbconnection,'ADD','NODE',$lang['GDPR_ADD_NODE']. $dev->getName() ,$lang['GDPR_ADD_NODE_FINALITY']);
		return $row->device_id;
	}
	private static function nodeObjectToArray(Device $dev)
	{
		$pass = $dev->getPassword();
		$arr = array(':name' => $dev->getName(),
			':type'=>$dev->getType() ,
			':ip'=>$dev->getIPAddress(),
			':login'=>$dev->getLogin(),
			':pwd'=>pg_escape_bytea($pass) ,
			':snmp_port'=>$dev->getSNMPObject()->getRemotePort(),
			':snmp_ver'=>$dev->getSNMPObject()->getVersion(),
			':snmp_timer'=>$dev->getSNMPObject()->getTimeout(),
			':community'=>$dev->getSNMPObject()->getCommunity(),
			':snmp3_user'=>$dev->getSNMPObject()->getSNMP3User(),
			':snmp3_seclvl'=>$dev->getSNMPObject()->getSNMP3SecLevel(),
			':snmp3_authprot'=>$dev->getSNMPObject()->getSNMP3AuthProtocol(),
			':snmp3_authkey'=>$dev->getSNMPObject()->getSNMP3AuthKey(),
			':snmp3_privprot'=>$dev->getSNMPObject()->getPrivacyProtocol(),
			':snmp3_privpass'=>$dev->getSNMPObject()->getPrivacyKey(),
			':protocol'=>$dev->getTypeConnection(),
			':grp'=>$dev->getGroup(),
			':description'=>$dev->getDescription()
			);
		if(!empty($dev->getId()))
		{
			$arr[':id']=$dev->getId();
		}
		return $arr;
	}
	/**
	 * autorise le group 'root' a accéder à se device
	 * @name addDeviceInDefaultGroup
	 * @param PDO $dbconnection
	 * @param integer|Device $dev L'objet Device à ajouter au groupe root ou l'id du device
	 * @throws EsiaDbException
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	//static function addAddDeviceInRootGroup($dbconnection,$dev )
	static function addDeviceInDefaultGroup($dbconnection,$dev )
	{
		if(@get_class($dev)=='Device')
		{
			$id =  $dev->getId();
		}
		elseif(@get_class($dev)=='RemoteDevice')
		{
			$id =  $dev->getId();
		}
		else
		{
			$id =  $dev;
		}
		$id =  $dev->getId();
		if(!empty( $id ))
		{
		    $query = $dbconnection->prepare("INSERT INTO esia_group_device_access VALUES (:group_id,:dev_id);");
			$result= $dbconnection->prepare("SELECT group_id FROM esia_groups WHERE group_name IN ('root','default_read_access');");
			$result = parent::executeStatement($result);
			while($row=$result->fetch(PDO::FETCH_OBJ))
			{
			     $query = parent::executeStatement($query,array(':group_id'=>$row->group_id,
					':dev_id' =>$dev->getId() ));
			}
		}
		else
		{
			throw new EsiaDbException("Id is NULL");
		}
	}

	/**
	 * Ajoute un nouveau plugin de test à esia
	 * @name addCheckPlugin
	 * @param PDO $dbconnection
	 * @param string $name Nom du plugin
	 * @param string $path Chemin d'accès à l'exécutable
	 * @param string $parameter paramètre par défaut
	 * @param integer $interval Interval de test
	 * @param string $description Description du plugin
	 * @param integer $priority  priorité du plugin (default 7)
	 * @param string $manufacturer fabricant du plugin
	 * @param string $tag tags du plugin
	 * @retval id du plugin nouvellement inséré
	 * @exception EsiaDbException si une erreur survient
	 */
	static function addCheckPlugin($dbconnection,$name,$path,$parameter,$interval,$description,$priority=7,$manufacturer,$tag)
	{
		$query= $dbconnection->prepare("INSERT INTO esia_plugins(plugin_id, plugin_name, plugin_path, plugin_type,
				plugin_default_parameter, plugin_default_interval_time, plugin_description, plugin_priority, plugin_manufacturer, plugin_tag)
				VALUES (DEFAULT, :name, :path, 'default', :param,:interval,:description,:priority, :manufacturer, :tag) RETURNING plugin_id;");
		
		$query = parent::executeStatement($query, array(
			':name' => $name,
			':path' => $path,
			':param' => $parameter,
			':interval' => $interval,
			':description' => $description,
			':priority' => $priority,
			':manufacturer' => $manufacturer,
			':tag' => $tag,
		));
		$row=$query->fetch(PDO::FETCH_OBJ);
		return $row->plugin_id;
	}

	/**
	 * Ajoute un nouveau plugin de traitement de trap à esia
	 * @name addTrapPlugin
	 * @param PDO $dbconnection
	 * @param string $name Nom du plugin
	 * @param string $path Chemin d'accès à l'exécutable
	 * @param string $parameter aramètre par défaut
	 * @param string $oid oid de la trap SNMP
	 * @param string $description Description du plugin
	 * @param integer $priority priorité du plugin
	 * @param string $manufacturer fabricant du plugin
	 * @param string $tag tags du plugin
	 * @retval id du plugin nouvellement inséré
	 * @exception EsiaDbException si une erreur survient
	 */
	static function addTrapPlugin($dbconnection,$name,$path,$parameter,$oid,$description,$priority=7,$manufacturer,$tag)
	{
		$query= $dbconnection->prepare("INSERT INTO esia_plugins(plugin_id, plugin_name, plugin_path, plugin_type,plugin_trap_oid,plugin_default_parameter,plugin_default_interval_time, plugin_description, plugin_priority, plugin_manufacturer, plugin_tag)
				VALUES (DEFAULT,:name,:path,'trap',:oid,:param,0,:description,:priority, :manufacturer, :tag) RETURNING plugin_id;");
		
		$query = parent::executeStatement($query, array(
			':name' => $name,
			':path' => $path,
			':param' => $parameter,
			':description' => $description,
			':oid' => $oid,
			':priority' => $priority,
			':manufacturer' => $manufacturer,
			':tag' => $tag,
		));

		$row=$query->fetch(PDO::FETCH_OBJ);
		return $row->plugin_id;
	}

	/**
	 * Ajoute un service à la base de donnée
	 * @name addService
	 * @param PDO $dbconnection
	 * @param string $name Nom du service
	 * @param integer $plugin id du plugin
	 * @param integer $node id du noeud
	 * @param integer $interval Interval de test
	 * @param string $parameter paramètre du service
	 * @param integer $priority priorité du service
	 * @param integer $alert id du programme a exécuter en cas d'erreur
	 * @param string $alert_param  = paramètre à utiliser
	 * @param bool $unknown si = 1 exécuter le programme d'alerte en cas d'erreur de type inconnu
	 * @param bool $warning si = 1 exécuter le programme d'alerte en cas d'erreur de type warning
	 * @param bool $critical si = 1 exécuter le programme d'alerte en cas d'erreur de type critical
	 * @throws EsiaDbException
	 * @retval l'identifiant du nouveau service
	 * @exception EsiaDbException si une erreur survient
	 */
	static function addService($dbconnection,$name,$plugin,$node,$interval,$parameter,$priority,$alert,$alert_param,$unknown=null,$warning=null,$critical=null,$ok=null)
	{
		if(is_numeric($node)&&is_numeric($plugin)&&is_numeric($interval))
		{
			//DEBUT GESA
			$gesa = Model_adm::getGesaLink($dbconnection,$node);
			if($gesa!=null)
			{
				if(gethostbyname($gesa->getName())==$gesa->getName())
				{
					$parameter = preg_replace('#\$NRPE_HOST#',$gesa->getIPAddress(),$parameter);
				}
				else
				{
					$parameter = preg_replace('#\$NRPE_HOST#',$gesa->getName(),$parameter);
				}
			}
			//FIN GESA
			$query= $dbconnection->prepare("INSERT INTO esia_services(service_id,service_name,service_plugin_id, service_device_id, service_interval_time,service_parameter,service_priority, service_previous_state)
					VALUES (DEFAULT,:name,:plugin_id,:device_id,:interval,:parameter,:priority,4) RETURNING service_id;");
			$query = parent::executeStatement($query,array(':name'=>$name,
					':plugin_id'=>$plugin,
					':device_id'=>$node,
					':interval'=>$interval,
					':parameter'=>$parameter,
					':priority'=>$priority));
			$serviceId = $query->fetch(PDO::FETCH_COLUMN);

			// si on a spécifier un programme d'alerte en parametre
			$alertParams = array();
			if($unknown!=NULL && $ok!=null && $warning!=null && $critical!=null && !empty($alert))
			{
				$alertParams = array(
					':service' => $serviceId,
					':program' => $alert,
					':parameter' => $alert_param,
					':warning' => $warning,
					':critical' => $critical,
					':ok' => $ok,
					':unknown' => $unknown,
				);
			}
			else
			{
				// Rechercher s'il y a un programme par defaut pour ce plugin
				$query = $dbconnection->prepare("
					SELECT react_program_id, react_parameter, react_ok, react_warning, react_critical, react_unknown
					FROM esia_plugin_react_program
					WHERE react_plugin_id=:plugin_id;
				");
				$query = Model_esia::executeStatement($query, array(':plugin_id' => $plugin));
				$pluginAlert = $query->fetch(PDO::FETCH_ASSOC);
				if(!empty($pluginAlert))
				{
					$alertParams = array(
						':service' => $serviceId,
						':program' => $pluginAlert['react_program_id'],
						':parameter' => $pluginAlert['react_parameter'],
						':warning' => empty($pluginAlert['react_warning']) ? 0 : 1,
						':critical' => empty($pluginAlert['react_critical']) ? 0 : 1,
						':ok' => empty($pluginAlert['react_ok']) ? 0 : 1,
						':unknown' => empty($pluginAlert['react_unknown']) ? 0 : 1,
					);
				}
			}
			// Ajouter le programme d'alerte trouver
			if(!empty($alertParams))
			{
				$query = $dbconnection->prepare("
					INSERT INTO esia_service_response_program(
					response_service_id, response_program_id, response_parameter, response_warning, response_critical, response_unknown,response_ok)
					VALUES (:service, :program, :parameter, :warning, :critical, :unknown, :ok);
				");
				$query = parent::executeStatement($query, $alertParams);
			}

			$query= $dbconnection->prepare("INSERT INTO esia_service_pending_execute(pending_service_id)VALUES (:id);");
			$query = parent::executeStatement($query, array(':id' => $serviceId));
			global $lang;
			model_adm::addGDPRRegister($dbconnection,'ADD','SERVICE',$lang['GDPR_ADD_SERVICE']. $name,$lang['GDPR_ADD_SERVICE_FINALITY']);

			return $serviceId;
		}
		else
		{
			throw new EsiaDbException('bad parameter: $plugin,$interval and $node must be numeric');
		}
	}

	/**
	 * ajoute les informations des graphiques
	 * @name addGraph
	 * @param PDO $dbconnection
	 * @param object|integer $node id du device ou un objet device
	 * @param string $url url d'accès au serveur munin/cacti
	 * @param integer $tree si cacti id de la tree au se trouve les graphs
	 * @param integer $leaf si cacti numéros de la variable leaf
	 * @throws EsiaDbException
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function addGraph($dbconnection, $node,$url,$tree=null,$leaf=null)
	{
		if(@get_class($node)=='Device')
		{
			$node =  $node->getId();
		}
		elseif(@get_class($node)=='RemoteDevice')
		{
			$node =  $node->getId();
		}
		else
		{
			$node =  $node;
		}
		if(!is_numeric($node))
		{
			throw new EsiaDbException('$node is not numeric or not object device');
		}
		if(!empty($tree) && !empty($leaf))
		{
			$query = $dbconnection->prepare("INSERT INTO esia_device_graph(graph_device_id, graph_url, graph_cacti_tree_id, graph_cacti_leaf_id, graph_software)
					VALUES (:node, :url, :tree, :leaf,'cacti');");
			$query = parent::executeStatement($query,array(':node'=>$node,
					':url'=>$url,
					':tree'=>$tree,
					':leaf'=>$leaf));

		}
		else
		{
			$query = $dbconnection->prepare("INSERT INTO esia_device_graph(graph_device_id, graph_url,graph_software)
					VALUES (:node, :url,'munin');");
			$query = parent::executeStatement($query,array(':node'=>$node,':url'=>$url));
		}

	}

	/**
	 * ajoute un nouvel utilisateur à l'application
	 * @name addUser
	 * @param PDO $dbconnection
	 * @param User $usermod un objet User à ajouter à la base de donnée
	 * @throws EsiaDbException
	 * @retval l'id de l'utilisateur
	 * @exception EsiaDbException si une erreur survient
	 */
	static function addUser($dbconnection,User $usermod)
	{
		if(! @get_class($usermod)=='User')
		{
			throw new EsiaDbException('$user is not a user\'s object');
		}
		$query=$dbconnection->prepare('INSERT INTO esia_users (user_id,user_name,user_password,user_change_password,user_email,user_lang,user_inscription_time,home_page)
				VALUES (default,:name,:password,1,:email,:lang,:now,:home_page) RETURNING user_id;');
		$query = parent::executeStatement($query,array(':name'=>$usermod->getName(),
				':password'=>$usermod->getHashPassword(),
				':email'=>$usermod->getEmail(),
				':lang'=>$usermod->getLang(),
				':now'=> $usermod->getInscriptionTime(),
				':home_page'=>$usermod->getHomePage()));
		$row=$query->fetch(PDO::FETCH_OBJ);
		global $lang;
		model_adm::addGDPRRegister($dbconnection,'ADD','USER',$lang['GDPR_ADD_USER']. $usermod->getName() ,$lang['GDPR_ADD_USER_FINALITY']);
		return $row->user_id;
	}

	/**
	 * ajoute l'utilisateur dans le groupe donné
	 * @name addUserInGroup
	 * @param PDO $dbconnection
	 * @param integer $user_id id de l'utilisateur
	 * @param integer $group_id id du group
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */

	static function addUserInGroup($dbconnection,$user_id,$group_id)
	{
		$query = $dbconnection->prepare("INSERT INTO esia_memberof VALUES (:group_id,:user_id);");
		$query = parent::executeStatement($query,array(':group_id'=>$group_id,
				':user_id'=>$user_id));
	}

	/**
	 * ajoute un groupe d'utilisateur à l'application
	 * @name addGroup
	 * @param PDO $dbconnection
	 * @param string $group nom du groupe
	 * @param string $description description du groupe
	 * @retval identifiant du groupe ajouté
	 * @exception EsiaDbException si une erreur survient
	 */
	static function addGroup($dbconnection,$group,$description)
	{
		$query = $dbconnection->prepare('INSERT INTO esia_groups VALUES (default,?,?) RETURNING group_id;');
		$query = parent::executeStatement($query,array($group,$description));
		$row=$query->fetch(PDO::FETCH_OBJ);
		return $row->group_id;
	}

	/**
	 * insére le pattern service dans la base de donnée
	 * @name addPatternService
	 * @param PDO $dbconnection
	 * @param string $name Nom du paternService
	 * @param string $description description du pattern service
	 * @param array $services tableau contenant les informations de base des plugins du pattern
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function addPatternService($dbconnection,$name,$description,$services=null)
	{
		$query = $dbconnection->prepare("INSERT INTO esia_service_patterns(pattern_id, pattern_name, pattern_description) VALUES (default, :name, :description) RETURNING pattern_id;");
		$query = parent::executeStatement($query,array(':name'=>$name,
				':description'=>$description));
		$row=$query->fetch(PDO::FETCH_OBJ);
		$pattern = $row->pattern_id;

		if(isset($services)&& is_array($services))
		{
			$query = $dbconnection->prepare("INSERT INTO esia_service_pattern_included(included_id,
																		included_name,
																		included_pattern_id,
																		included_plugin_id,
																		included_parameter,
																		included_interval,
																		included_priority )
					VALUES (DEFAULT, :name, :pattern,:plugin,:parameter,:interval,:priority) RETURNING included_id ;");
			$queryUpdate = $dbconnection->prepare("UPDATE esia_service_pattern_included
													SET included_program_id=:program,
  														included_program_parameter=:alertparameter,
  														included_unknown=:unknown,
														included_warning=:warning,
														included_critical=:critical,
														included_ok=:ok
													WHERE included_id = :id;");

			foreach($services as $ser)
			{
				$query = parent::executeStatement($query,array(':name'=>$ser['name'],
						':pattern'=>$pattern,
						':plugin'=>$ser['plugin'],
						':parameter'=>$ser['parameter'],
						':interval'=>$ser['interval'],
						':priority'=>$ser['priority']));
				if(!empty($ser['alerting_software']))
				{
					$row=$query->fetch(PDO::FETCH_OBJ);
					$params = array(
						':program'=>$ser['alerting_software'],
						':alertparameter'=>	$ser['alerting_parameter'],
						':id'=>$row->included_id
					);
					foreach(array('unknown','warning','critical','ok') as $key)
					{
						if(  $ser[$key] == TRUE )
						{
							$params[":$key"] = 't';
						}
						else
						{
							$params[":$key"] = 'f';
						}
					}
					$queryUpdate = parent::executeStatement($queryUpdate,$params);
				}
			}
		}
		return $pattern;
	}

	static function searchIncludedService($search,$list,$key_idx=0)
	{
		$keys=array(
			'plugin_id','name','priority','interval','parameter','alerting','alerting_parameter','unknown','warning','critical','ok'
		);

		$key = $keys[$key_idx];
		$tmp = array();
		foreach($list as $row)
		{
			if( $search[$key] == $row[$key] )
			{
				$tmp[] =  $row;
			}
		}

		$key_idx++;
		if( count($tmp) > 1 && isset($keys[$key_idx]))
		{
			return self::searchIncludedService($search,$tmp,$key_idx);
		}
		return array_shift($tmp);
	}

	/**
	 * Déplace un ensemble de noeuds d'un pattern vers un autre pattern.
	 * @name moveNodesToPatternServices
	 * @param PDO $dbconnection
	 * @param string $old_pattern Nom de l'ancien pattern
	 * @param string $new_pattern Nom du nouveau pattern
	 * @param array $nodes_id_list tableau contenant la liste des id des noeuds
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function moveNodesToPatternServices($dbconnection,$old_pattern,$new_pattern,$nodes_id_list)
	{
		if( empty($nodes_id_list) ) throw new Exception("Missing or empty nodes list");

		$pattern_id = $old_pattern;
		$pattern = Model_adm::getPatternService($dbconnection,$pattern_id);
		$pattern = $pattern[$pattern_id];
		$old_plugins = $pattern['plugins'];

		$pattern_id = $new_pattern;
		$pattern = Model_adm::getPatternService($dbconnection,$pattern_id);
		$pattern = $pattern[$pattern_id];
		$new_plugins = $pattern['plugins'];

		$old_to_new_list = array();
		foreach($old_plugins as $old)
		{
			$new = self::searchIncludedService($old,$new_plugins);
			$old_to_new_list[$old['id']] = $new['id'];
		}

		//Vérification contre les SQL INJECTION car on bypasse la vérification PDO ici.
		$validated_id = array();
		foreach($nodes_id_list as $node_id)
		{
			if ( FALSE !== filter_var($node_id, FILTER_VALIDATE_INT, array("options" => array("min_range"=>1)))) {
    			$validated_id[] = $node_id;
			}
			else
			{
				if( empty($nodes_id_list) ) throw new Exception("Invalid node id");
			}
		}

		//Tous les noeuds included_id par included_id
		$sql = "UPDATE esia_services SET service_included_id = :new_id WHERE service_included_id = :old_id";
		$sql.= ' AND service_device_id IN('.implode(',',$validated_id).')';
		$query = $dbconnection->prepare($sql);

		foreach($old_to_new_list as $old_id => $new_id)
		{
			$params[':new_id'] = $new_id;
			$params[':old_id'] = $old_id;
			$query = parent::executeStatement($query,$params);
		}
	}

	/**
	 * Déplace un ensemble de noeuds d'un pattern vers un autre pattern.
	 * @name moveServicesNodeToPatternServices
	 * @param PDO $dbconnection
	 * @param array $services_list Liste des services à déplacer vers un nouveau pattern
	 * @param string $new_pattern Nom du nouveau pattern
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function moveServicesNodeToPatternServices($dbconnection,$services_list,$new_pattern)
	{
		$services_list = Model_esia::getServices($dbconnection,NULL,$services_list,NULL,TRUE);
		$old_services = array();
		foreach($services_list as $row)
		{
			$retRow = array();
			$retRow['id'] = $row['id'];
			$retRow['plugin_id'] = $row['plugin_id'];
			$retRow['name'] = $row['name'];
			$retRow['priority'] = $row['priority'];
			$retRow['interval'] = $row['interval_time'];
			$retRow['parameter'] = $row['parameter'];
			$retRow['alerting'] = $row['alerting_program_id'];
			$retRow['alerting_parameter'] = $row['alerting_program_parameter'];
			$retRow['unknown'] = $row['alerting_program_unknown'];
			$retRow['warning'] = $row['alerting_program_warning'];
			$retRow['critical'] = $row['alerting_program_critical'];
			$retRow['ok'] = $row['alerting_program_ok'];
			$old_services[] = $retRow;
		}

		$pattern_id = $new_pattern;
		$pattern = Model_adm::getPatternService($dbconnection,$pattern_id);
		$pattern = $pattern[$pattern_id];
		$new_plugins = $pattern['plugins'];

		$old_to_new_list = array();
		foreach($old_services as $old)
		{
			$new = self::searchIncludedService($old,$new_plugins);
			$old_to_new_list[$old['id']] = $new['id'];
		}

		//Tous les noeuds included_id par included_id
		$sql = "UPDATE esia_services SET service_included_id = :new_id WHERE service_id = :old_id";
		$query = $dbconnection->prepare($sql);

		foreach($old_to_new_list as $old_id => $new_id)
		{
			$params[':new_id'] = $new_id;
			$params[':old_id'] = $old_id;
			$query = parent::executeStatement($query,$params);
		}
	}

	static function makeIncludedFromServices($dbconnection,$name,$description,$services_list)
	{
		$services_list = Model_esia::getServices($dbconnection,NULL,$services_list,NULL,TRUE);
        foreach($services_list as $key => $value)
        {
            $services_list[$key]['plugin'] = $value['plugin_id'];
            $services_list[$key]['interval'] = $value['interval_time'];
        }
		return Model_adm::addPatternService($dbconnection,$name,$description,$services_list);
	}

	/**
	 * ajoute un groupe de noeud  à l'application
	 * @name addGroupNode
	 * @param PDO $dbconnection
	 * @param string $group nom du groupe
	 * @param integer $parent groupe parent
	 * @retval identifiant du groupe de noeud ajouté
	 * @exception EsiaDbException si une erreur survient
	 */
	static function addGroupNode($dbconnection,$group,$parent=null,$description=NULL)
	{
		if(isset($parent))
		{
			$query = $dbconnection->prepare('INSERT INTO esia_device_group(device_group_id, device_group_name, device_group_parent,device_group_description)
											 VALUES (DEFAULT, ?, ?, ?) RETURNING device_group_id;');
			$query = parent::executeStatement($query,array($group,$parent,$description));
		}
		else
		{
			$query = $dbconnection->prepare('INSERT INTO esia_device_group(device_group_id, device_group_name, device_group_parent, device_group_description)
					VALUES (DEFAULT, ?, NULL, ?) RETURNING device_group_id;');
			$query = parent::executeStatement($query,array($group,$description));
		}
		$row=$query->fetch(PDO::FETCH_OBJ);
		return $row->device_group_id;
	}

	/**
	 * Assigne un pattern de service à un noeud
	 * @name assignPatternServiceNode
	 * @param PDO $dbconnection
	 * @param integer $patternid identifiant du pattern de service
	 * @param integer $node identifiant du noeud
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function assignPatternServiceNode($dbconnection,$patternid,$node)
	{
		$pattern = Model_adm::getPatternService($dbconnection,$patternid);
		$pattern = $pattern[$patternid];
		$query = $dbconnection->prepare("UPDATE esia_services SET service_included_id=:patternid WHERE service_id = :serid");
		foreach($pattern['plugins'] as $plugin)
		{
			//on ajoutou modifie un service
			$warning='FALSE';
			$critical='FALSE';
			$unknown='FALSE';
			$ok='FALSE';
			if(!empty($plugin['unknwon']))
				$unknown='TRUE';
			if(!empty($plugin['warning']))
				$warning='TRUE';
			if(!empty($plugin['critical']))
				$critical='TRUE';
			if(!empty($plugin['ok']))
				$ok='TRUE';
			$serid = model_adm::addService($dbconnection,$plugin['name'],$plugin['plugin_id'],$node,
								$plugin['interval'],$plugin['parameter'],$plugin['priority'],$plugin['alerting'],$plugin['alerting_parameter'],
								$unknown,$warning,$critical,$ok);
			$query = parent::executeStatement($query,array(':serid'=>$serid,':patternid'=>$plugin['id']));
		}
	}

	/**
	 * met à jour l'objet device
	 * @name updateDevice
	 * @param PDO $dbconnection
	 * @param object Device $dev
	 * @throws Exception
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function updateDevice($dbconnection,Device $dev)
	{
		$id =  $dev->getId();
		if(!empty( $id ))
		{
			$query = $dbconnection->prepare("UPDATE esia_device SET device_name=:name ,
					device_type=:type ,
					device_ip_address=:ip,
					device_protocol_connection=:protocol,
					device_username=:login,
					device_passwd=:pwd,
					device_description =:description,
					device_snmp_version=:snmp_ver,
					device_snmp_timer=:snmp_timer,
					device_snmp_community=:community,
					device_snmp3_user=:snmp3_user,
					device_snmp3_sec_lvl=:snmp3_seclvl,
					device_snmp3_auth_protocole=:snmp3_authprot,
					device_snmp3_auth_key=:snmp3_authkey,
					device_snmp3_priv_protocol=:snmp3_privprot,
					device_snmp3_priv_passphrase=:snmp3_privpass,
					device_snmp_port=:snmp_port,
					device_group_name=:grp
					WHERE device_id=:id;");
			$arr = Model_adm::nodeObjectToArray($dev);
			$query = parent::executeStatement($query,$arr);
			//suppression des anciens groupes
			$query = $dbconnection->prepare("DELETE FROM esia_device_group_in WHERE group_in_device_id=:id");
			$query = parent::executeStatement($query,array(':id' => $dev->getId()));
			//ajout dans les nouveaux groupes
			$query = $dbconnection->prepare("INSERT INTO esia_device_group_in(group_in_device_id, group_in_device_group_id)VALUES (:id, :group);");
			$grps =  $dev->getGroupList();
			foreach($grps as $grp)
			{
				$query = parent::executeStatement($query,array(':id' => $dev->getId(),':group'=>$grp));
			}
		}
		else
		{
			throw new Exception("Id is NULL");
		}
	}

	/**
	 * met à jours les informations des graphiques
	 * @name updateGraph
	 * @param PDO $dbconnection
	 * @param object|integer $node id du device ou un objet device
	 * @param string $url url d'accès au serveur munin/cacti
	 * @param integer $tree si cacti id de la tree au se trouve les graphs
	 * @param integer $leaf si cacti numéros de la variable leaf
	 * @throws EsiaDbException
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function updateGraph($dbconnection, $node,$url,$tree=null,$leaf=null)
	{
		if(@get_class($node)=='Device')
		{
			$node =  $node->getId();
		}
		elseif(@get_class($node)=='RemoteDevice')
		{
			$node =  $node->getId();
		}
		else
		{
			$node =  $node;
		}
		if(!is_numeric($node))
		{
			throw new EsiaDbException('$node is not numeric or not object device');
		}
		if(!empty($tree) && !empty($leaf))
		{
			$query = $dbconnection->prepare("UPDATE esia_device_graph
					SET graph_url=:url, graph_cacti_tree_id=:tree, graph_cacti_leaf_id=:leaf, graph_software='cacti'
					WHERE graph_device_id=:node;");
			$query = parent::executeStatement($query,array(':node'=>$node,
					':url'=>$url,
					':tree'=>$tree,
					':leaf'=>$leaf));
		}
		else
		{
			$query = $dbconnection->prepare("UPDATE esia_device_graph
					SET graph_url=:url, graph_software='munin'
					WHERE graph_device_id=:node;");
			$query = parent::executeStatement($query,array(':node'=>$node,':url'=>$url));
		}
	}

	/**
	 * met à jour un plugin de test
	 * @name UpdateCheckPlugin
	 * @param PDO $dbconnection
	 * @param integer $id id du plugin
	 * @param string $name Nom du plugin
	 * @param string $path Chemin d'accès à l'exécutable
	 * @param string $parameter paramètre par défaut
	 * @param integer $interval Interval de test
	 * @param string $description Description du plugin
	 * @param integer $priority priorité du plugin
	 * @param string $manufacturer fabricant du plugin
	 * @param string $tag tags du plugin
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function UpdateCheckPlugin($dbconnection,$id,$name,$path,$parameter,$interval,$description,$priority,$manufacturer,$tag)
	{
		$query = $dbconnection->prepare("UPDATE esia_plugins SET plugin_name=:name,plugin_path=:path,plugin_type='default',
				plugin_default_parameter=:param, plugin_default_interval_time=:interval, plugin_description=:description,plugin_priority=:priority, plugin_manufacturer=:manufacturer, plugin_tag=:tag
				WHERE plugin_id=:id;");

		parent::executeStatement($query, array(
			':name' => $name,
			':path' => $path,
			':param' => $parameter,
			':interval' => $interval,
			':description' => $description,
			':id' => $id,
			':priority' => $priority,
			':manufacturer' => $manufacturer,
			':tag' => $tag,
		));
	}

	/**
	 * met à jour un plugin de traitement de trap
	 * @name UpdateTrapPlugin
	 * @param PDO $dbconnection
	 * @param integer $id id du plugin
	 * @param string $name Nom du plugin
	 * @param string $path Chemin d'accès à l'exécutable
	 * @param string $parameter paramètre par défaut
	 * @param string $oid oid de la trap SNMP
	 * @param string $description Description du plugin
	 * @param integer $priority priorité du plugin
	 * @param string $manufacturer fabricant du plugin
	 * @param string $tag tags du plugin
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function UpdateTrapPlugin($dbconnection,$id,$name,$path,$parameter,$oid,$description,$priority,$manufacturer,$tag)
	{
		$query= $dbconnection->prepare("UPDATE esia_plugins SET plugin_name=:name,plugin_path=:path,plugin_type='trap',plugin_trap_oid=:oid,
				plugin_default_parameter=:param, plugin_description=:description,plugin_priority=:priority, plugin_manufacturer=:manufacturer, plugin_tag=:tag
				WHERE plugin_id=:id");
				
		parent::executeStatement($query, array(
			':name' => $name,
			':path' => $path,
			':param' => $parameter,
			':id' => $id,
			':description' => $description,
			':oid' => $oid,
			':priority' => $priority,
			':manufacturer' => $manufacturer,
			':tag' => $tag,
		));
	}

	/**
	 * Mets à jour les paramètres d'un service
	 * @name UpdateService
	 * @param PDO $dbconnection
	 * @param integer $id id du service
	 * @param string $name Nom du service
	 * @param integer $plugin id du plugin
	 * @param integer $node id du noeud
	 * @param integer $interval Interval de test
	 * @param string $parameter paramètre du service
	 * @param integer $priority priorité du service
	 * @param integer $alert id du programme a exécuter en cas d'erreur
	 * @param string $alert_param  = paramètre à utiliser
	 * @param bool $unknown si = 1 exécuter le programme d'alerte en cas d'erreur de type inconnu
	 * @param bool $warning si = 1 exécuter le programme d'alerte en cas d'erreur de type warning
	 * @param bool $critical si = 1 exécuter le programme d'alerte en cas d'erreur de type critical
	 * @throws EsiaDbException
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function UpdateService($dbconnection,$id,$name,$plugin,$node,$interval,$parameter,$priority,$alert,$alert_param=null,$unknown=null,$warning=null,$critical=null,$ok=null)
	{
		if(is_numeric($node)&&is_numeric($plugin)&&is_numeric($interval))
		{
			//DEBUT GESA
			$gesa = Model_adm::getGesaLink($dbconnection,$node);
			if($gesa!=null)
			{
				if(gethostbyname($gesa->getName())==$gesa->getName())
				{
					$parameter = preg_replace('#\$NRPE_HOST#',$gesa->getIPAddress(),$parameter);
				}
				else
				{
					$parameter = preg_replace('#\$NRPE_HOST#',$gesa->getName(),$parameter);
				}
			}
			//FIN GESA
			$query= $dbconnection->prepare("UPDATE esia_services
					SET service_plugin_id=:plugin_id, service_interval_time=:interval,
					service_parameter=:parameter, service_name=:name,service_priority=:priority
					WHERE service_id=:id;");
			$query = parent::executeStatement($query,array(':name'=>$name,
					':plugin_id'=>$plugin,
					':id'=>$id,
					':interval'=>$interval,
					':parameter'=>$parameter,
					':priority'=>$priority));
            if($alert !=-1)
            {
			    $query= $dbconnection->prepare("DELETE FROM esia_service_response_program WHERE response_service_id=:id;");
			    $query = parent::executeStatement($query,array(':id'=>$id));

			    //si on a spécifier un programme d'alerte
			    if($ok!==null && $warning!==null && $critical!==null && $unknown!==NULL && (!empty($alert)||$alert!=0) )
			    {
				    $query= $dbconnection->prepare("INSERT INTO esia_service_response_program(
						    response_service_id, response_program_id, response_parameter, response_warning, response_critical, response_unknown,response_ok)
						    VALUES (:service, :program, :parameter,:warning, :critical, :unknown, :ok);");
				    $query = parent::executeStatement($query,array(':service'=>$id,
						    ':program'=>$alert,
						    ':parameter'=>$alert_param,
						    ':warning'=>$warning,
						    ':critical'=>$critical,
						    ':ok'=>$ok,
						    ':unknown'=>$unknown));
			    }
            }
			Model_adm::reloadService($dbconnection,$node,$id);
		//	$query= $dbconnection->prepare("INSERT INTO esia_service_pending_execute(pending_service_id,pending_type)VALUES (:id,'UPDATE');");
		//	$query = parent::executeStatement($query,array(':id'=>$id));

		}
		else
		{
			throw new EsiaDbException('bad parameter: $plugin,$inteval and $node must be numeric');
		}
	}
	/**
	 * @brief reload les services demandé au niveau du démon de traitement
	 * @param PDO $dbconnection  variable de connexion à la DB
	 * @param integer $nodeid identifiant du noeud
	 * @param integer $serviceid identifiant du service (optionnel) si il n'est spécifé alors l'ensemble des services sont rechargés
	 * @throws EsiaDbException si une erreur survient
	 */
	static function reloadService($dbconnection,$nodeid,$serviceid=null)
	{
		try
		{
			if(is_numeric($nodeid)&&is_numeric($serviceid))
			{

				$query= $dbconnection->prepare("INSERT INTO esia_service_pending_execute(pending_service_id,pending_type)VALUES (:id,'UPDATE');");
				$query = parent::executeStatement($query,array(':id'=>$serviceid));

			}
			elseif(is_numeric($nodeid))
			{
				$query= $dbconnection->prepare("INSERT INTO esia_service_pending_execute(
												pending_service_id, pending_type)
												SELECT service_id,'UPDATE' FROM esia_services WHERE service_device_id=:id;");
				$query = parent::executeStatement($query,array(':id'=>$nodeid));
			}
			else
			{
				throw new EsiaDbException('reload service bad parameter: $nodeid must be numeric');
			}
		}
		catch (EsiaDbException $e)
		{
			if(! preg_match('#pending_primary_key#', $e->getMessage()))
			{
				throw new EsiaDbException($e->getMessage());
			}
		}
	}

	/**
	 * Mets à jour le mot de passe de l'utilisateur
	 * @name UpdateUserPassword
	 * @param PDO $dbconnection
	 * @param User $usermod objet User contenant l'utilisateur
	 * @param string $password le nouveau mot de passe
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function UpdateUserPassword($dbconnection, User $usermod,$password)
	{
		$hashpassword = $usermod->generateHashPassword($usermod->getName(),$password,$usermod->getInscriptionTime());
		$query = $dbconnection->prepare('UPDATE esia_users SET user_password=:pass,user_change_password=1 WHERE user_id=:id;');
		$query = parent::executeStatement($query,array(':id'=>$usermod->getId(),
				':pass'=>$hashpassword));
	}

	/**
	 * Mets à jour les informations de l'utilisateurs
	 * @name UpdateUser
	 * @param PDO $dbconnection
	 * @param User $usermod objet User contenant l'utilisateur
	 * @param string $password (optionnel) le nouveau mot de passe
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function UpdateUser($dbconnection, User $usermod,$password=null)
	{
		//$hashpassword = $usermod->generateHashPassword($usermod->getName(),$password,$usermod->getInscriptionTime());
		$query = $dbconnection->prepare('UPDATE esia_users
				SET user_change_password=1,
				user_email = :mail,
				user_lang = :lang,
				home_page = :home_page
				WHERE user_id=:id;');
		$query = parent::executeStatement($query,array(':id'=>$usermod->getId(),
				':mail'=>$usermod->getEmail(),
				':home_page'=>$usermod->getHomePage(),
				':lang'=>$usermod->getLang()));
	}

	/**
	 * Mets à jour les droits d'accès d'un groupe, attention les anciens droits sont écrasés
	 * @name UpdateGroupGrantAccess
	 * @param PDO $dbconnection
	 * @param integer $groupid id du group
	 * @param array $access tableau contenant les identifiants des nouveaux droit d'accès
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function UpdateGroupGrantAccess($dbconnection, $groupid,$access)
	{
		$query = $dbconnection->prepare("DELETE FROM esia_privilege_grantedto WHERE grant_group_id=:group_id;");
		$query = parent::executeStatement($query,array(':group_id'=>$groupid));
		$query = $dbconnection->prepare("INSERT INTO esia_privilege_grantedto(grant_group_id, grant_privilege_id, grant_acces) VALUES ( :group_id , :access_id ,1);");
		foreach($access as $selectdroitId)
		{
			if($selectdroitId!=0&&is_numeric($selectdroitId))
			{
				$query = parent::executeStatement($query,array(':group_id'=>$groupid,
						':access_id'=>$selectdroitId,));
			}
		}
	}

	/**
	 *  Mets à jour les droits d'accès d'un groupe par rapport au noeud, attention les anciens droits sont écrasés
	 * @name UpdateGroupNodeAccess
	 * @param PDO $dbconnection
	 * @param integer $groupid id du group
	 * @param array $access tableau contenant les identifiants des noeuds où le groupe à le droit d'accès
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function UpdateGroupNodeAccess($dbconnection, $groupid,$access)
	{
		$query = $dbconnection->prepare("DELETE FROM esia_group_device_access WHERE access_group_id=:group_id;");
		$query = parent::executeStatement($query,array(':group_id'=>$groupid));
		//$query = $dbconnection->prepare("INSERT INTO esia_group_device_access VALUES ( :group_id , :dev_id);");

		$listnode = array();
		foreach($access as $selectdroitId)
		{
			if($selectdroitId!=0&&is_numeric($selectdroitId))
			{
			    $listnode[]=$selectdroitId;
			}
		}
		if(!empty($listnode))
		{
    		$listnode = implode(',', $listnode);
    		$query = $dbconnection->prepare("INSERT INTO esia_group_device_access
                                            SELECT :group_id,device_id
                                            FROM esia_device
                                            WHERE device_id IN(".$listnode.");");
            $query = parent::executeStatement($query,array(':group_id'=>$groupid));
		}
	}

	/**
	 * met à jour un groupe de noeud  à l'application
	 * @name updateGroupNode
	 * @param PDO $dbconnection
	 * @param integer $id id du groupe
	 * @param integer $group id du groupe
	 * @param integer $parent groupe parent
	 * @retval identifiant du groupe de noeud ajouté
	 * @exception EsiaDbException si une erreur survient
	 */
	static function updateGroupNode($dbconnection,$id,$group,$parent=null,$description = NULL)
	{
		if(isset($parent))
		{
			$query = $dbconnection->prepare('UPDATE esia_device_group
											   SET device_group_name=:name, device_group_parent=:parent, device_group_description=:description
											 WHERE device_group_id=:id;');
			$query = parent::executeStatement($query,array(':id'=>$id,':name'=>$group,':parent'=>$parent,':description'=>$description));
		}
		else
		{
			$query = $dbconnection->prepare('UPDATE esia_device_group
											   SET device_group_name=:name, device_group_parent=NULL, device_group_description=:description
											 WHERE device_group_id=:id;');
			$query = parent::executeStatement($query,array(':id'=>$id,':name'=>$group,':description'=>$description));
		}
		return $id;
	}

	/**
	 * insére le pattern service dans la base de donnée
	 * @name updatePatternService
	 * @param PDO $dbconnection
	 * @param integer $id identifiant du pattern
	 * @param string $name Nom du paternService
	 * @param string $description description du pattern service
	 * @param array $services tableau contenant les informations de base des plugins du pattern
	 *		$array[]['id'] identifiant du services (included_id dans la table services_included)
	 *		$array[]['name'] nouveau nom du service
	 *		$array[]['plugin'] id du plugin
	 *		$array[]['parameter] nouveau paramètre
	 *		$array[]['interval'] nouvel interval
	 * @param string $nodes_also Pour update aussi les noeuds. Appel à la méthode updateNodesPatternServices()
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function updatePatternService($dbconnection,$id,$name,$description,$services=null,$nodes_also=TRUE)
	{
		//mise à jour des informations du pattern
		$query = $dbconnection->prepare('UPDATE esia_service_patterns SET
											pattern_name=:name,
											pattern_description=:description
										WHERE pattern_id=:id;');
		$query = parent::executeStatement($query,array(':id'=>$id,
														':name'=>$name,
														':description'=>$description));
		//on récupère la liste des noeuds / services associé à ce pattern
		$query = $dbconnection->prepare("SELECT service_id,service_device_id,included_id
										  FROM esia_services
										  JOIN esia_service_pattern_included ON service_included_id=included_id
										  WHERE included_pattern_id=:id;");
		$query = parent::executeStatement($query,array(':id'=>$id));
		$arr= array();
		$node_arr= array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$arr[$row->included_id][]=array('service_id'=>$row->service_id,'device_id'=>$row->service_device_id);
			$node_arr[$row->service_device_id]=$row->service_device_id;
		}
		//préparation pour l'update des service inclus au pattern
		$query = $dbconnection->prepare("UPDATE esia_service_pattern_included SET
											included_name = :name,
											included_parameter=:parameter,
											included_interval=:interval,
											included_program_id=:program,
  											included_program_parameter=:alertparameter,
  											included_unknown=:unknown,
											included_warning=:warning,
											included_critical=:critical,
											included_ok=:ok,
											included_priority=:priority
										WHERE included_id = :id;");
		$qinsert = $dbconnection->prepare("INSERT INTO esia_service_pattern_included(included_id, included_name, included_pattern_id, included_plugin_id, included_parameter, included_interval,included_priority)
				VALUES (default, :name, :pattern,:plugin,:parameter,:interval,:priority) RETURNING included_id;");

		foreach($services as $ser)
		{
			if(!empty($ser['id']))
			{
				if(!empty($ser['alerting_software']) && $ser['alerting_software']!=0)
				{
					parent::executeStatement($query,array(':id'=>$ser['id'],
																	':name'=>$ser['name'],
																	':parameter'=>$ser['parameter'],
																	':interval'=>$ser['interval'],
																	':program'=>$ser['alerting_software'],
																	':alertparameter'=>	$ser['alerting_parameter'],
																	':unknown'=>$ser['unknown'],
																	':warning'=> $ser['warning'],
																	':critical'=>$ser['critical'],':ok'=>$ser['ok'],
																	':priority'=>$ser['priority'],));
				}
				else
				{
					parent::executeStatement($query,array(':id'=>$ser['id'],
							':name'=>$ser['name'],
							':parameter'=>$ser['parameter'],
							':interval'=>$ser['interval'],
							':program'=>null,
							':alertparameter'=>	null,
							':unknown'=>null,
							':warning'=>null,
							':critical'=>null,
							':ok'=>null,
							':priority'=>$ser['priority'],));
				}
			}
			else
			{
				$rinsert= parent::executeStatement($qinsert,array(':name'=>$ser['name'],
						':pattern'=>$id,
						':plugin'=>$ser['plugin'],
						':parameter'=>$ser['parameter'],
						':interval'=>$ser['interval'],':priority'=>$ser['priority']));
				$row=$rinsert->fetch(PDO::FETCH_OBJ);
				$partternid = $row->included_id;
			}
		}

		//Update des services des noeuds si nécessaire (ancien fonctionnement/Fonctionnement par défaut)
		if($nodes_also)
		{
			Model_Adm::updateNodesPatternServices($dbconnection,$id,$node_arr,NULL);
		}
	}

	/**
	 * Assigne ou met à jour les services d'un patter à un ensemble de noeuds
	 * @name updateNodesPatternServices
	 * @param PDO $dbconnection
	 * @param integer $pattern_id identifiant du pattern
	 * @param array $nodes_id_list List des noeuds à mettre à jour
	 * @param array $services tableau contenant les informations de base des plugins du pattern
	 *		$array[]['id'] identifiant du services (included_id dans la table services_included)
	 *		$array[]['name'] nouveau nom du service
	 *		$array[]['plugin'] id du plugin
	 *		$array[]['parameter] nouveau paramètre
	 *		$array[]['interval'] nouvel interval
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function updateNodesPatternServices($dbconnection,$pattern_id,$nodes_id_list=NULL,$services=NULL)
	{
		if( $services === NULL )
		{
			$pattern = Model_adm::getPatternService($dbconnection,$pattern_id);
			$pattern = $pattern[$pattern_id];
			$services = array();
			foreach($pattern['plugins'] as $row)
			{
				$services[] = $row;
			}
		}
		foreach($services as $key => $row)
		{
			if( isset($row['plugin_id']))
			{
				$services[$key]['plugin'] = $row['plugin_id'];
			}
		}

		//on récupère la liste des noeuds / services associé à ce pattern
		$query = $dbconnection->prepare("SELECT service_id,service_device_id,included_id
										  FROM esia_services
										  JOIN esia_service_pattern_included ON service_included_id=included_id
										  WHERE included_pattern_id=:id;");


		$query = parent::executeStatement($query,array(':id'=>$pattern_id));
		$arr= array();
		$node_arr= array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			if( $nodes_id_list === NULL || in_array($row->service_device_id,$nodes_id_list))
			{
				$arr[$row->included_id][]=array('service_id'=>$row->service_id,'device_id'=>$row->service_device_id);
				$node_arr[$row->service_device_id]=$row->service_device_id;
			}
		}

		$qupdate = $dbconnection->prepare("UPDATE esia_services SET service_included_id=:patternid WHERE service_id = :serid");

		foreach($services as $ser)
		{
			if( $ser['alerting'] > 0)
			{
				foreach(array('ok','warning','critical','unknown') as $value)
				{
					if(  $ser[$value] === TRUE )
					{
						$ser[$value] = 't';
					}
					else
					{
						$ser[$value] = 'f';
					}
				}
			}

			$list = $arr[$ser['id']];
			foreach($node_arr as $node_id)
			{
				$found = FALSE;
				foreach($list as $row)
				{
					if( $row['device_id'] == $node_id )
					{
						try
						{
							Model_adm::UpdateService($dbconnection,$row['service_id'],$ser['name'],$ser['plugin'],$node_id,
							$ser['interval'],$ser['parameter'],$ser['priority'],$ser['alerting'],$ser['alerting_parameter']
							,$ser['unknown'],$ser['warning'],$ser['critical'],$ser['ok']);
						}
						catch (EsiaDbException $e)
						{
							print_r($e);
						}
						$found = TRUE;
						break;
					}

				}
				if( !$found )
				{
					try {
						$serid =Model_adm::addService($dbconnection, $ser['name'], $ser['plugin'], $node_id, $ser['interval'], $ser['parameter'],$ser['priority'],$ser['alerting'], $ser['alerting_parameter'],
																	 $ser['unknown'],$ser['warning'],$ser['critical'],$ser['ok']);
						parent::executeStatement($qupdate,array(':serid'=>$serid,':patternid'=>$ser['id']));
					}
					catch (EsiaDbException $e)
					{
						print_r($e);
					}
				}
			}
		}
	}

	/**
	 * supprime le noeud de l'application
	 * @name deleteNode
	 * @param PDO $dbconnection
	 * @param string|integer $nodeid id du device ou son nom ou un objet device
	 * @throws Exception
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function deleteNode($dbconnection,$nodeid)
	{
		if(@get_class($nodeid)=='Device')
		{
			$nodeid =  $nodeid->getId();
		}
		elseif(@get_class($nodeid)=='RemoteDevice')
		{
			$nodeid =  $nodeid->getId();
		}
		if(is_numeric($nodeid))
		{
			$query = $dbconnection->prepare("UPDATE esia_device SET device_delete=TRUE WHERE device_id =?;");
		}
		elseif(is_string($nodeid))
		{
		    $query = $dbconnection->prepare("UPDATE esia_device SET device_delete=TRUE WHERE device_name =?;");
		}
		else
		{
			throw new Exception('$nodeid must be numeric or equals device name');
		}
		global $lang;
		model_adm::addGDPRRegister($dbconnection,'DELETE','NODE',$lang['GDPR_DELETE_NODE']. $nodeid ,$lang['GDPR_DELETE_NODE_FINALITY']);
		$query = parent::executeStatement($query,array($nodeid));
	}

	/**
	 * supprime le plugin de l'application
	 * @name deletePlugin
	 * @param PDO $dbconnection
	 * @param integer $pluginid id du plugin
	 * @throws EsiaDbException
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function deletePlugin($dbconnection, $pluginid)
	{
		if(is_numeric($pluginid))
		{
			$params = array(':id' => $pluginid);
			$query = $dbconnection->prepare("UPDATE esia_services SET service_delete=TRUE
					WHERE service_plugin_id=:id");
			$query = parent::executeStatement($query, $params);
			$query = $dbconnection->prepare("UPDATE esia_plugins SET plugin_delete=TRUE
					WHERE plugin_id=:id");
			$query = parent::executeStatement($query, $params);
			// Delete alerting plugin default configuration
			$query = $dbconnection->prepare("DELETE FROM esia_plugin_react_program WHERE react_plugin_id=:id");
			$query = parent::executeStatement($query, $params);
		}
		else
		{
			throw new EsiaDbException('plugin\'s id is not numeric');
		}
	}

	/**
	 * supprime le service de l'application
	 * @name deleteService
	 * @param PDO $dbconnection
	 * @param integer $service id du service
	 * @throws EsiaDbException
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function deleteService($dbconnection,$service)
	{
		if(is_numeric($service))
		{
			$query= $dbconnection->prepare("DELETE FROM esia_error_happen
					WHERE happen_service_id=?");
			$query = parent::executeStatement($query,array($service));
			$query= $dbconnection->prepare("UPDATE esia_services SET service_delete=TRUE
					WHERE service_id=?");
			$query = parent::executeStatement($query,array($service));
			global $lang;
			model_adm::addGDPRRegister($dbconnection,'DELETE','SERVICE',$lang['GDPR_DELETE_SERVICE']. $service,$lang['GDPR_DELETE_SERVICE_FINALITY']);

		}
		else
		{
			throw new EsiaDbException('service is not numeric');
		}
	}
	/**
	 * supprime l'utilisateur de l'application
	 * @name deleteUser
	 * @param PDO $dbconnection
	 * @param integer $id id de l'utilisateur
	 * @throws EsiaDbException
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function deleteUser($dbconnection,$id)
	{
		$query = $dbconnection->prepare("SELECT member_user_id FROM esia_memberof WHERE member_user_id=?;");
		$query = parent::executeStatement($query,array($id));
		$nb_rows = $query->rowCount();
		//si la requête renvoi un nombre de ligne différent de 0 alors
		//on supprime la liaison de l'utilisateur avec le groupe
		if($nb_rows!=0)
		{
			$query = $dbconnection->prepare("DELETE FROM esia_memberof WHERE member_user_id=?;");
			$query = parent::executeStatement($query,array($id));
		}
		//suppression de l'utilisateur
		$query = $dbconnection->prepare("DELETE FROM esia_users WHERE user_id=?;");
		$query = parent::executeStatement($query,array($id));
		global $lang;
		model_adm::addGDPRRegister($dbconnection,'DELETE','USER',$lang['GDPR_DELETE_USER']. $id,$lang['GDPR_DELETE_USER_FINALITY']);

	}

	/**
	 * retire l'utilisateur d'un groupe donnée
	 * @name deleteUserFromGroup
	 * @param PDO $dbconnection
	 * @param integer $userid id ou le nom de l'utilisateur
	 * @param integer $groupid id ou le nom du group
	 * @throws EsiaDbException
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function deleteUserFromGroup($dbconnection,$userid,$groupid)
	{
		if(!is_numeric($userid))
		{
			$userid = Model_adm::InstanceUser($dbconnection, $userid);
			$userid = $userid->getId();
		}
		if(!is_numeric($groupid))
		{
			$query = $dbconnection->prepare("SELECT group_id FROM esia_groups WHERE group_name=?;");
			$query = parent::executeStatement($query,array( $groupid ));
			$row=$query->fetch(PDO::FETCH_OBJ);
			$groupid=$row->group_id;
		}
		$query = $dbconnection->prepare("DELETE FROM esia_memberof WHERE member_user_id=? AND member_group_id=?;");
		$query = parent::executeStatement($query,array($userid,$groupid));

	}

	/**
	 * supprime le groupe passé en paramètre
	 * @name deleteGroup
	 * @param PDO $dbconnection
	 * @param integer $id id du groups
	 * @throws EsiaDbException
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function deleteGroup($dbconnection,$id)
	{
		$query = $dbconnection->prepare("DELETE FROM esia_memberof WHERE member_group_id=?;");
		$query = parent::executeStatement($query,array($id));
		$query = $dbconnection->prepare("DELETE FROM esia_group_device_access WHERE access_group_id=?;");
		$query = parent::executeStatement($query,array($id));
		$query = $dbconnection->prepare("DELETE FROM esia_privilege_grantedto WHERE grant_group_id=?");
		$query = parent::executeStatement($query,array($id));
		$query = $dbconnection->prepare("DELETE FROM esia_groups WHERE group_id=?;");
		$query = parent::executeStatement($query,array($id));
	}

	/**
	 * supprime le service du pattern ainsi que tout les service actif qui si rapporte
	 * @name deleteServiceOfPattern
	 * @param PDO $dbconnection
	 * @param integer $id id du service (table included_id)
	 * @throws EsiaDbException
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function deleteServiceOfPattern($dbconnection,$id)
	{
		$query = $dbconnection->prepare("SELECT service_id FROM esia_services WHERE service_included_id=?;");
		$query = parent::executeStatement($query,array($id));
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			Model_adm::deleteService($dbconnection, $row->service_id);
		}
		$query =  $dbconnection->prepare("UPDATE esia_services SET service_included_id=NULL WHERE service_included_id=?;");
		$query = parent::executeStatement($query,array($id));
		$query =  $dbconnection->prepare("DELETE FROM esia_service_pattern_included WHERE included_id=?;");
		$query = parent::executeStatement($query,array($id));

	}

	/**
	 * supprime le pattern de service ainsi que tout les services actifs associés
	 * @name deletePatternService
	 * @param PDO $dbconnection
	 * @param integer $id id du pattern de service
	 * @throws EsiaDbException
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function deletePatternService($dbconnection,$id)
	{
		$query = $dbconnection->prepare("SELECT service_id,included_id
										FROM esia_services
										JOIN esia_service_pattern_included ON included_id = service_included_id
										WHERE included_pattern_id=?;");
		$query = parent::executeStatement($query,array($id));
		$list = array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			Model_adm::deleteService($dbconnection, $row->service_id);
			$list[$row->included_id]=$row->included_id;
		}
		$query =  $dbconnection->prepare("UPDATE esia_services SET service_included_id=NULL WHERE service_included_id=?;");
		foreach($list as $inc_id)
		{
			parent::executeStatement($query,array($inc_id));
		}
		$query =  $dbconnection->prepare("DELETE FROM esia_service_pattern_included WHERE included_pattern_id=?;");
		$query = parent::executeStatement($query,array($id));
		$query =  $dbconnection->prepare("DELETE FROM esia_service_patterns WHERE pattern_id=?;");
		$query = parent::executeStatement($query,array($id));
	}

	/**
	 * supprime le groupe de noeud passé en paramètre et les groups enfant devienne des groupes membres de la racine principal
	 * @name deleteGroupNode
	 * @param PDO $dbconnection
	 * @param integer $id id du groups
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function deleteGroupNode($dbconnection,$id)
	{
		$query = $dbconnection->prepare('UPDATE esia_device_group SET device_group_parent=NULL
											 WHERE device_group_parent=?;');
		$query = parent::executeStatement($query,array($id));
		$query = $dbconnection->prepare("DELETE FROM esia_device_group_in WHERE group_in_device_group_id=?");
		$query = parent::executeStatement($query,array($id));
		$query = $dbconnection->prepare("DELETE FROM esia_device_group WHERE device_group_id=?;");
		$query = parent::executeStatement($query,array($id));
	}

	/**
	 * test et si il y a une gesa qui est liée elle retourne un objet device
	 * @name getGesaLink
	 * @param PDO $dbconnection
	 * @param integer $nodeid identifiant du noeud
	 * @retval NULL|GESA null si aucune gesa n'est lié sinon un objet device
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getGesaLink($dbconnection,$nodeid)
	{
		$query= $dbconnection->prepare("SELECT connected_gesa_id FROM gesa_connected WHERE connected_device_id=?;");
		$query = parent::executeStatement($query,array($nodeid));
		$gesa=null;
		if($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$gesa = new GESA();
			$gesa->constructDevice(Model_esia::InstanceNode($dbconnection,$row->connected_gesa_id));
		}
		return $gesa;
	}

	/**
	 * Lie la GESA au noeud correspondant
	 * @name linkGesaNode
	 * @param PDO $dbconnection
	 * @param integer $gesaid identifiant de la GESA
	 * @param integer $nodeid identifiant du noeud
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function linkGesaNode($dbconnection,$gesaid,$nodeid)
	{
		$query= $dbconnection->prepare("INSERT INTO gesa_connected(connected_gesa_id, connected_device_id) VALUES (?, ?);");
		$query = parent::executeStatement($query,array($gesaid,$nodeid));
	}

	/**
	 * Délie la GESA au noeud correspondant
	 * @name unlinkGesaNode
	 * @param PDO $dbconnection
	 * @param integer $gesaid identifiant de la GESA
	 * @param integer $nodeid identifiant du noeud
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function unlinkGesaNode($dbconnection,$gesaid,$nodeid)
	{
		$query= $dbconnection->prepare("DELETE FROM gesa_connected WHERE connected_gesa_id =? AND connected_device_id=?;");
		$query = parent::executeStatement($query,array($gesaid,$nodeid));
	}

	/**
	 * Dissocie un pattern de service du noeud donné. Supprime les services!!!!!!
	 * @name unassignPatternServiceNode
	 * @param PDO $dbconnection
	 * @param integer $patternid identifiant du pattern de service
	 * @param integer $node identifiant du noeud
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function unassignPatternServiceNode($dbconnection,$patternid,$node)
	{
		$query = $dbconnection->prepare("SELECT service_id
											FROM esia_services
											JOIN esia_service_pattern_included ON service_included_id = included_id
											WHERE included_pattern_id=:id AND service_device_id=:node;");
		$query = parent::executeStatement($query,array(':id'=>$patternid,':node'=>$node));
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			Model_adm::deleteService($dbconnection, $row->service_id);
		}
	}
	/**
	 * Délie un pattern de service du noeud donné. Ne supprime pas les services!!!!!!
	 * @name unlinkPatternServiceNode
	 * @param PDO $dbconnection
	 * @param integer $patternid identifiant du pattern de service
	 * @param integer $node identifiant du noeud
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function unlinkPatternServiceNode($dbconnection,$patternid,$node)
	{
		$query = $dbconnection->prepare("UPDATE esia_services SET service_included_id = NULL WHERE service_id IN (SELECT service_id
											FROM esia_services
											JOIN esia_service_pattern_included ON service_included_id = included_id
											WHERE included_pattern_id=:id AND service_device_id=:node);");
		$query = parent::executeStatement($query,array(':id'=>$patternid,':node'=>$node));
	}
	/**
	 * recherche un utilisateur ou tout les utilisateur
	 * @name searchUser
	 * @param PDO $dbconnection
	 * @param string|integer $username (optionnel) l identifiant ou le nom ou une partie du nom de l'utilisateur (si pas préciser retourne tout les utilisateurs
	 * @retval un tableau associatif
	 *	$arr[<user id>]['id'] id de l'utilisateur
	 *	$arr[<user id>]['name'] nom de l'utilisateur
	 *	$arr[<user id>]['mail'] adresse mail de l'utilisateur
 	 *	$arr[<user id>]['group'] tableau contenant la liste des groupes auquel l'utilisateur appartient
	 * @exception EsiaDbException si une erreur survient
	 */
	static function searchUser($dbconnection,$username=null)
	{
		if(isset($username))
		{
			if(is_numeric($username))
			{
				$query = $dbconnection->prepare("SELECT user_id,user_name,user_email,group_name,group_id
						FROM esia_users
						LEFT JOIN esia_memberof ON user_id = member_user_id
						LEFT JOIN esia_groups ON group_id = member_group_id
						WHERE user_id = ?;");
				$query = parent::executeStatement($query,array($username));
			}
			else
			{
				$query = $dbconnection->prepare("SELECT user_id,user_name,user_email,group_name,group_id
						FROM esia_users
						LEFT JOIN esia_memberof ON user_id = member_user_id
						LEFT JOIN esia_groups ON group_id = member_group_id
						WHERE user_name LIKE ?;");
				$query = parent::executeStatement($query,array('%'.$username.'%'));
			}
		}
		else
		{
			$query = $dbconnection->prepare("SELECT user_id,user_name,user_email,group_name,group_id
					FROM esia_users
					LEFT JOIN esia_memberof ON user_id = member_user_id
					LEFT JOIN esia_groups ON group_id = member_group_id
					WHERE visibility_user = TRUE;");
			$query = parent::executeStatement($query);
		}
		$list_user=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$list_user[$row->user_id]['id'] = $row->user_id;
			$list_user[$row->user_id]['name']=$row->user_name;
			$list_user[$row->user_id]['mail'] = $row->user_email;
			$list_user[$row->user_id]['group'][]=$row->group_name;
			$list_user[$row->user_id]['group_id'][]=$row->group_id;
		}
		return $list_user;
	}

	/**
	 * retourne les groupes dont l'utilisateur n'est pas membre
	 * @name getUserNotInGroup
	 * @param PDO $dbconnection
	 * @param string|integer $userid (optionnel) l'identifiant ou le nom de l'utilisateur
	 * @retval un tableau associatif
	 *	$arr[<group id>]['id'] id du groupe
	 *	$arr[<group id>]['name'] nom du groupe
	 * @exception EsiaDbException si une erreur survient
	 */
	static function getUserNotInGroup($dbconnection,$userid)
	{
		$query = $dbconnection->prepare("SELECT group_id,group_name
				FROM esia_groups
				EXCEPT
				SELECT group_id,group_name
				FROM esia_groups
				JOIN esia_memberof ON group_id=member_group_id
				WHERE member_user_id=?");
		$query = parent::executeStatement($query,array( $userid ));
		$list_grp=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$list_grp[$row->group_id]['id'] = $row->group_id;
			$list_grp[$row->group_id]['name']=$row->group_name;
		}
		return $list_grp;
	}

	/**
	 * met à jour la configuration
	 * @name updateConfiguration
	 * @param PDO $dbconnection
	 * @param array $arr tableau associatifs ayant comme structure $list_key[<clé>]=valeur
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function updateConfiguration($dbconnection,$arr)
	{
		$query = $dbconnection->prepare("UPDATE esia_configuration SET config_value=:value WHERE config_key=:key");
		foreach($arr as $key => $value)
		{
			$query = parent::executeStatement($query,array('value'=>$value,'key'=>$key));
		}
	}

	/**
	 * indique qu'il faut recharger la file de service
	 * @name mustReloadDaemon
	 * @param PDO $dbconnection
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function mustReloadDaemon($dbconnection)
	{
		$query= $dbconnection->prepare("UPDATE esia_configuration SET config_value=1 WHERE config_key='SYS_RELOAD';");
		$query = parent::executeStatement($query);
	}

	/**
	 * indique qu'il faut redémarrer le programme
	 * @name mustRestartDaemon
	 * @param PDO $dbconnection
	 * @retval /
	 * @exception EsiaDbException si une erreur survient
	 */
	static function mustRestartDaemon($dbconnection)
	{
		$query= $dbconnection->prepare("UPDATE esia_configuration SET config_value=2 WHERE config_key='SYS_RELOAD';");
		$query = parent::executeStatement($query);
	}
	/**
	 * @brief met à jour une tuile
	 * @param PDO $dbconnection
	 * @param integer $id  identifiant de la DB (table: dashboard_tile)
	 * @param integer $position la position par rapport au dashboard
	 * @param integer $height la hauteur de la tuile
	 * @param integer $width la largeur de la tuile
	 * @param integer $dashboardid identifiant du dashboard
	 * @exception: une esiaDBException
	 */
	static function updateTileDashboard($dbconnection,$id,$position,$height,$width,$dashboardid)
	{
		$query = $dbconnection->prepare("UPDATE dashboard_tile
										   SET tile_position=:position, tile_height=:height,
												tile_width=:width, tile_dashboard_id=:dashid
										 WHERE tile_id=:id");
		$query = parent::executeStatement($query,array('id'=>$id,
														'position'=>$position,
														'height'=>$height,
														'width'=>$width,
														'dashid'=>$dashboardid));
	}
	/**
	 * @brief ajoute une nouvelle tuille
	 * @param PDO $dbconnection
	 * @param integer $position la position par rapport au dashboard
	 * @param integer $height la hauteur de la tuile
	 * @param integer $width la largeur de la tuile
	 * @param integer $dashboardid identifiant du dashboard
	 * @retval integer identifiant de la tuile en DB (table: dashboard_tile)
	 * @exception: une esiaDBException
	 */
	static function insertTileDashboard($dbconnection,$position,$height,$width,$dashboardid)
	{
		$query = $dbconnection->prepare("INSERT INTO dashboard_tile(
            tile_id, tile_position, tile_height, tile_width, tile_dashboard_id)
			    VALUES (DEFAULT,:position,:height,:width,:dashid) RETURNING tile_id;");
		$query = parent::executeStatement($query,array(	'position'=>$position,
				'height'=>$height,
				'width'=>$width,
				'dashid'=>$dashboardid));
		$row=$query->fetch(PDO::FETCH_OBJ);
		return $row->tile_id;
	}
	/**
	 * @brief récupère la liste des utilisateur a qui on a partagé le dashboard
	 * @param PDO $dbconnection
	 * @param integer $id identifiant du dashboard
	 * @exception: esiaDBException
	 */
	static function getUserShareDashboard($dbconnection,$id,$write=false)
	{
		if($write==false)
		{
			$query = $dbconnection->prepare("SELECT board_user_id
  											FROM dashboard_user_access
											WHERE (board_user_share = true OR board_user_access='READ')
												AND board_dashboard_id=?;");
		}
		else
		{
			$query = $dbconnection->prepare("SELECT board_user_id
  											FROM dashboard_user_access
											WHERE (board_user_share = false OR board_user_access='WRITE')
												AND board_dashboard_id=?;");
		}
		$query = parent::executeStatement($query,array($id));
		$list=array();
		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
			$list[]=$row->board_user_id;
		}
		return $list;
	}
	/**
	 * @brief met à jour un dashboard
	 * @param PDO $dbconnection
	 * @param integer $id identifiant du dashboard
	 * @param integer $name nom du dashboard
	 * @param integer $description  description du dashboard
	 * @exception: une esiaDBException
	 */
	static function updateDashboard($dbconnection,$id,$name,$description,$users=null,$write=null,$userid=null)
	{
		$query = $dbconnection->prepare("UPDATE dashboard
 				  SET dashboard_name=:name, dashboard_description=:description WHERE dashboard_id=:id;");
		$query = parent::executeStatement($query,array(	'id'=>$id,'name'=>$name,'description'=>$description));
	//	$row=$query->fetch(PDO::FETCH_OBJ);

		if(isset($write))
		{
			$queryDel = $dbconnection->prepare("DELETE FROM dashboard_user_access WHERE board_dashboard_id = ? AND board_user_id = ? AND board_user_share = true;");
			$query = $dbconnection->prepare("DELETE FROM dashboard_user_access WHERE board_dashboard_id = ? AND board_user_id <> ? AND board_user_share = FALSE;");
			$query = parent::executeStatement($query,array($id,$userid));
			$query = $dbconnection->prepare("INSERT INTO dashboard_user_access(board_user_id, board_dashboard_id, board_user_share,board_user_access) VALUES (?,?,FALSE,'WRITE');");
			foreach($write as $w)
			{
			    try {
			        $queryDel = parent::executeStatement($queryDel,array($id,$w));
			        $query = parent::executeStatement($query,array($w,$id));
			    } catch (Exception $e) {
			    }
			}
		}
		else
		{
			$query = $dbconnection->prepare("DELETE FROM dashboard_user_access WHERE board_dashboard_id = ? AND board_user_id <> ? AND board_user_access = 'WRITE';");
			$query = parent::executeStatement($query,array($id,$userid));
		}

		if(isset($users))
		{
			$query = $dbconnection->prepare("DELETE FROM dashboard_user_access WHERE board_dashboard_id = ? AND board_user_access = 'READ';");
			$query = parent::executeStatement($query,array($id));
			$query = $dbconnection->prepare("INSERT INTO dashboard_user_access(board_user_id, board_dashboard_id, board_user_share,board_user_access) VALUES (?,?,TRUE,'READ');");
			foreach($users as $user)
			{
				$query = parent::executeStatement($query,array($user,$id));
			}
		}
		else
		{
			$query = $dbconnection->prepare("DELETE FROM dashboard_user_access WHERE board_dashboard_id = ? AND board_user_share = true;");
			$query = parent::executeStatement($query,array($id));
		}
		//return $row->tile_id;
	}
	/**
	 * @brief Crée un lien d'accès public au tableau de bord
	 * @param PDO $dbconnection
	 * @param integer $id identifiant du dashboard
	 * @param integer $userid identification de l'utilisateur
	 * @exception: une esiaDBException
 	 * @retval string la chaine d'accès public du tableau de bord
	 */
	static function generateDashboardPublicLink($dbconnection,$id,$userid,$onlydashboard = "")
	{
		$date = new DateTime();
		$str = 'ID:'.$id.';userid'.$userid.';date:'.$date->getTimestamp();
		$str = sha1($str);
		if(!empty($onlydashboard))
		{
			$str.='-'.$onlydashboard;
		}
		$query = $dbconnection->prepare("DELETE FROM dashboard_user_access WHERE board_dashboard_id = ? AND board_user_access = 'LINK';");
		$query = parent::executeStatement($query,array($id));
		$query = $dbconnection->prepare("INSERT INTO dashboard_user_access(board_user_id, board_dashboard_id, board_user_share,board_user_access) VALUES (?,?,TRUE,'LINK');");
		$query = parent::executeStatement($query,array($userid,$id));
		$query = $dbconnection->prepare("UPDATE dashboard SET dashboard_link=? WHERE dashboard_id=?;");
		$query = parent::executeStatement($query,array($str,$id));
		return $str;
	}
	/**
	 * @brief Supprime le lien public d'accès au tableu de bord
	 * @param PDO $dbconnection
	 * @param integer $id identifiant du dashboard
	 * @exception: une esiaDBException
	 * @retval string l'identifiant public du tableau de bord
	 */
	static function deleteDashboardPublicLink($dbconnection,$id)
	{
		$query = $dbconnection->prepare("DELETE FROM dashboard_user_access WHERE board_dashboard_id = ? AND board_user_access = 'LINK';");
		$query = parent::executeStatement($query,array($id));
		$query = $dbconnection->prepare("UPDATE dashboard SET dashboard_link=NULL WHERE dashboard_id=?;");
		$query = parent::executeStatement($query,array($id));
	}
	/**
	 * @brief retourne le lien public
	 * @param PDO $dbconnection
	 * @param integer $id identifiant du dashboard
	 * @param string $hashlink la chaine d'accès
	 * @exception: une esiaDBException
	 * @retval array array('id'=> $row->dashboard_id,'linkid' => $row->dashboard_link,'userid' => $row->board_user_id);
	 */
	static function getDashboardPublicLink($dbconnection,$id=null,$hashlink=null)
	{
		$sql = "SELECT dashboard_id,dashboard_link,board_user_id
					FROM dashboard
					JOIN dashboard_user_access ON board_dashboard_id=dashboard_id";
		$params = array();
		if(!empty($id))
		{
			$sql .= " WHERE dashboard_id = ? AND board_user_access='LINK';";
			$params[] = $id;
		}
		else
		{
			$sql .= " WHERE dashboard_link = ? AND board_user_access='LINK';";
			$params[] = $hashlink;
		}
		$arr = array();
		$query = $dbconnection->prepare($sql);
		$query = parent::executeStatement($query,$params);
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$arr[] = array('id'=> $row->dashboard_id,'linkid' => $row->dashboard_link,'userid' => $row->board_user_id);
		}
		return $arr[0];
	}
	/**
	 * @brief ajoute un nouveau dashboard
	 * @param PDO $dbconnection
	 * @param integer $id identifiant du dashboard
	 * @param integer $nom du dashboard
	 * @param integer $description description
	 * @exception: une esiaDBException
     * @retval integer identifiant du dashboard en DB (table: dashboard)
	 */
	static function insertDashboard($dbconnection,$name,$description,$controller,$userid)
	{
		$query = $dbconnection->prepare("INSERT INTO dashboard(
            dashboard_id, dashboard_name, dashboard_controller, dashboard_description)
    								VALUES (DEFAULT,:name,:controller,:description)  RETURNING dashboard_id;");
		$query = parent::executeStatement($query,array('name'=>$name,'description'=>$description,'controller'=>$controller));
		$row=$query->fetch(PDO::FETCH_OBJ);

		$query = $dbconnection->prepare("INSERT INTO dashboard_user_access(board_user_id, board_dashboard_id, board_user_share)
										 VALUES (:userid, :dashid, FALSE);");
		$query = parent::executeStatement($query,array('userid'=>$userid,'dashid'=>$row->dashboard_id));

		return $row->dashboard_id;
	}
	/**
	 * @brief Duplique les widget & les tuiles d'un dashboard vers un autre
	 * @param PDO $dbconnection
	 * @param integer $idori identifiant du dashboard d'origine
	 * @param integer $idnew identifiant du dashboard de destination
	 * @param integer $userid identifiant de l'utilisateur createur des dashboards
	 * @exception: une esiaDBException
	 * @retval none
	 */
	static function duplicateDashboard($dbconnection,$idori,$idnew,$userid)
	{
		$dashboardorigine = model_esia::getDashboards($dbconnection,$idori,$userid);
		$dashboardorigine = array_pop($dashboardorigine);

		//Copie des tuilles
		$query = $dbconnection->prepare("INSERT INTO dashboard_tile(tile_position, tile_height, tile_width, tile_dashboard_id)
										SELECT tile_position, tile_height, tile_width, :idnew
										FROM dashboard_tile
										WHERE tile_dashboard_id=:idori
										ORDER BY tile_position;");
		//$sth->bindValue(':idori', $idori, PDO::PARAM_INT);
		//$sth->bindValue(':idori', $idori, PDO::PARAM_INT);
		$query = parent::executeStatement($query,array('idori'=>$idori,'idnew'=>$idnew));
		$row=$query->fetch(PDO::FETCH_OBJ);
		$dashboardnew = model_esia::getDashboards($dbconnection,$idnew,$userid);
		$dashboardnew = array_pop($dashboardnew);	
		$tilesori = $dashboardorigine['tile'];
		$tilesnew = $dashboardnew['tile'];
		
		$query = $dbconnection->prepare("INSERT INTO dashboard_widget(widget_name, widget_view_during, widget_position, widget_option, 
																	  widget_tile_id, widget_view_id, widget_description)
										SELECT widget_name, widget_view_during, widget_position, widget_option,:newtile, widget_view_id, widget_description
										FROM dashboard_widget
										WHERE widget_tile_id=:idori
										ORDER BY widget_position;");
		$tnew = current($tilesnew);
		foreach( $tilesori as $tori)
		{			
			$query = parent::executeStatement($query,array(':idori'=>$tori['id'],':newtile'=>$tnew['id']));
			$tnew = next($tilesnew);
		}
		return $idnew;
	}
	/**
	 * @brief supprime un dashboard
	 * @param PDO $dbconnection
	 * @param integer $id identifiant du dashboard
	 * @exception: une esiaDBException
	 * @retval /
	 */
	static function deleteDashboard($dbconnection,$id)
	{
		$query = $dbconnection->prepare("DELETE FROM dashboard_widget WHERE widget_tile_id IN (SELECT tile_id FROM dashboard_tile WHERE tile_dashboard_id=?);");
		$query = parent::executeStatement($query,array($id));
		$query = $dbconnection->prepare("DELETE FROM dashboard_tile WHERE tile_dashboard_id = ?;");
		$query = parent::executeStatement($query,array($id));
		$query = $dbconnection->prepare("DELETE FROM dashboard_user_access WHERE board_dashboard_id = ?;");
		$query = parent::executeStatement($query,array($id));
		$query = $dbconnection->prepare("DELETE FROM dashboard WHERE dashboard_id = ?;");
		$query = parent::executeStatement($query,array($id));
	}
	/**
	 * @brief supprime une tuile du dashboard
	 * @param PDO $dbconnection
	 * @param integer $id identifiant de la tuile
	 * @exception: une esiaDBException
	 * @retval /
	 */
	static function deleteTileDashboard($dbconnection,$id)
	{
		$query = $dbconnection->prepare("DELETE FROM dashboard_widget WHERE widget_tile_id =?;");
		$query = parent::executeStatement($query,array($id));
		$query = $dbconnection->prepare("DELETE FROM dashboard_tile WHERE tile_id = ?;");
		$query = parent::executeStatement($query,array($id));
	}
	/**
	 * @brief supprime une widget
	 * @param PDO $dbconnection
	 * @param integer $id identifiant de la widget
	 * @exception: une esiaDBException
	 * @retval /
	 */
	static function deleteWidgetDashboard($dbconnection,$id)
	{
		$query = $dbconnection->prepare("DELETE FROM dashboard_widget WHERE widget_id =?;");
		$query = parent::executeStatement($query,array($id));
	}
	/**
	 * @brief ajoute une nouvelle tuille
	 * @param PDO $dbconnection
	 * @param integer $tileid identifiant de la tuile
	 * @param integer $viewid identifiant de la vue
	 * @retval integer identifiant de la widget en DB (table: dashboard_tile)
	 * @exception: une esiaDBException
	 */
	static function insertWidget($dbconnection,$tileid,$viewid,$name='new widget')
	{
		$query = $dbconnection->prepare("INSERT INTO dashboard_widget( widget_id, widget_name, widget_view_during,widget_option, widget_tile_id, widget_view_id)
   				 VALUES (DEFAULT,:name,
						(SELECT view_during FROM dashboard_view WHERE view_id=:viewid),
						(SELECT view_default_option FROM dashboard_view WHERE view_id=:viewid),:tileid,:viewid) RETURNING widget_id;");
		$query = parent::executeStatement($query,array(	'viewid'=>$viewid,'tileid'=>$tileid,'name'=>$name ));
		$row=$query->fetch(PDO::FETCH_OBJ);
		return $row->widget_id;
	}
	/**
	 * @brief ajoute une nouvelle tuille
	 * @param PDO $dbconnection
	 * @param integer $tileid identifiant de la tuile
	 * @param integer $viewid identifiant de la vue
	 * @retval integer identifiant de la widget en DB (table: dashboard_tile)
	 * @exception une esiaDBException
	 */
	static function updateWidgetOrder($dbconnection,$array)
	{
		$query = $dbconnection->prepare("UPDATE dashboard_widget SET widget_position=:position WHERE widget_id=:id;");
		foreach($array as $line)
		{
			$query = parent::executeStatement($query,array(	'id'=>$line['id'],'position'=>$line['position']));
		}
	}
    static function addServiceMaintenancePlan($dbconnection,$serviceid,$type, $start,$end,$during)
    {
        $query = $dbconnection->prepare("INSERT INTO esia_maintenance_planned(planned_id, planned_type, planned_start, planned_end, planned_during,planned_service_id)
                                        VALUES (DEFAULT,:type,to_timestamp(:start,'DD-MM-YYYY HH24:MI'),to_timestamp(:end,'DD-MM-YYYY HH24:MI'),:during,:service);");
        if(is_array($serviceid))
        {
            foreach($serviceid as $id)
            {
                if(is_numeric($id))
                {
                    $query = parent::executeStatement($query,array(	'type'=>$type,'service'=>$id,'start'=>$start->format('d-m-Y H:i'),'end'=>$end->format('d-m-Y H:i'),'during'=>$during ));
                }
            }

        }
        elseif(is_numeric($serviceid))
        {
            $query = parent::executeStatement($query,array(	'type'=>$type,'service'=>$serviceid,'start'=>$start->format('d-m-Y H:i'),'end'=>$end->format('d-m-Y H:i'),'during'=>$during ));
        }
        else
        {
            throw new EsiaDbException('$serviceid must be numeric or array');
        }
    }
    static function deleteServiceMaintenancePlan($dbconnection,$plannedid)
    {
        if(is_numeric($plannedid))
        {
            $query = $dbconnection->prepare("DELETE FROM esia_maintenance_planned WHERE planned_id =?");
            $query = parent::executeStatement($query,array($plannedid));
        }
        else
        {
             throw new EsiaDbException('$plannedid must be numeric');
        }
    }
    static function getNextMaintenancePlan($dbconnection,$serviceid=null,$node=null )
    {
        $sql ='SELECT planned_id, planned_type, planned_start, planned_end, planned_during,planned_service_id ,service_name,plugin_name,device_id,device_name
               FROM esia_maintenance_planned
               JOIN esia_services ON service_id = planned_service_id
               JOIN esia_plugins ON plugin_id = service_plugin_id
               JOIN esia_device ON device_id = service_device_id ';
        if(is_numeric($serviceid))
        {
            $query = $dbconnection->prepare($sql."WHERE planned_service_id=? AND planned_start > NOW();");
            $query = parent::executeStatement($query,array($serviceid));
        }
        elseif(is_numeric($node))
        {
            $query = $dbconnection->prepare($sql."WHERE service_device_id=?  AND planned_start > NOW();");
                        $query = parent::executeStatement($query,array($node));
        }
        elseif(is_array($node))
        {
		    foreach($node as $id)
			{
				if(!is_numeric($id))
					throw new EsiaDbException('$node array must be numeric');
			}
            $arrids = implode(',', $node);
            $query = $dbconnection->prepare($sql."WHERE service_device_id IN ($arrids)   AND planned_start > NOW();");
                        $query = parent::executeStatement($query);
        }
        else
        {
            throw new EsiaDbException('$service or $node must be numeric');
        }

        $arr = array();
  		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
            $arr[] = array('id'=>$row->planned_id,'type'=>$row->planned_type, 'start'=>$row->planned_start,'end'=>$row->planned_end,'during'=>$row->planned_during,'service'=>$row->planned_service_id,
                    'service_name'=>$row->service_name,
                    'plugin_name'=>$row->plugin_name,
                    'device_name'=>$row->device_name,
                    'device_id'=>$row->device_id );
        }
        return $arr;
    }

    static function addCyclicMaintenance($dbconnection,$serviceid,$type, $start,$during,$interval,$endfirsterror = null )
    {
    	$sql ="INSERT INTO esia_maintenance_configuration( maintenance_conf_id, maintenance_conf_type, maintenance_conf_start_date,
    			maintenance_conf_create, maintenance_conf_service_id, maintenance_conf_during, maintenance_conf_recurring,maintenance_conf_during_firsterror )
    			VALUES (DEFAULT,:type,to_timestamp(:start,'DD-MM-YYYY HH24:MI'),NOW(), :service ,cast(:during as interval),cast(:interval as interval),cast(:firsterrorduring as interval) );";
    	$query = $dbconnection->prepare($sql);
    	if(is_array($serviceid))
    	{
    		foreach($serviceid as $id)
    		{
    			if(is_numeric($id))
    			{
    				$query = parent::executeStatement($query,array(	'type'=>$type,'service'=>$id,'start'=>$start->format('d-m-Y H:i'),'during'=>$during,':interval'=>$interval,':firsterrorduring'=>$endfirsterror ));
    			}
    		}

    	}
    	elseif(is_numeric($serviceid))
    	{
    		$query = parent::executeStatement($query,array(	'type'=>$type,'service'=>$id,'start'=>$start->format('d-m-Y H:i'),'during'=>$during,':interval'=>$interval,':firsterrorduring'=>$endfirsterror ));
    	}
    	else
    	{
    		throw new EsiaDbException('$serviceid must be numeric or array');
    	}
    }

    static function getCyclicMaintenancePlan($dbconnection,$serviceid=null,$node=null )
    {
        $sql ='SELECT maintenance_conf_id, maintenance_conf_type, maintenance_conf_start_date,maintenance_conf_create, maintenance_conf_service_id,
                    maintenance_conf_during, maintenance_conf_recurring,service_name,plugin_name,device_id,device_name
               FROM esia_maintenance_configuration
               JOIN esia_services ON service_id = maintenance_conf_service_id
               JOIN esia_plugins ON plugin_id = service_plugin_id
               JOIN esia_device ON device_id = service_device_id ';
        if(is_numeric($serviceid))
        {
            $query = $dbconnection->prepare($sql."WHERE planned_service_id=?");
            $query = parent::executeStatement($query,array($serviceid));
        }
        elseif(is_numeric($node))
        {
            $query = $dbconnection->prepare($sql."WHERE service_device_id=?");
                        $query = parent::executeStatement($query,array($node));
        }
        elseif(is_array($node))
        {
		    foreach($node as $id)
			{
				if(!is_numeric($id))
					throw new EsiaDbException('$node array must be numeric');
			}
            $arrids = implode(',', $node);
            $query = $dbconnection->prepare($sql."WHERE service_device_id IN ($arrids);");
                        $query = parent::executeStatement($query);
        }
        else
        {
            throw new EsiaDbException('$service or $node must be numeric');
        }
        $arr = array();
  		while($row=$query->fetch(PDO::FETCH_OBJ))
		{
            $arr[] = array('id'=>$row->maintenance_conf_id,'type'=>$row->maintenance_conf_type,
                    'start'=>$row->maintenance_conf_start_date,'create'=>$row->maintenance_conf_create,'during'=>$row->maintenance_conf_during,'service'=>$row->maintenance_conf_service_id,
                    'interval'=>$row->maintenance_conf_recurring,
                    'service_name'=>$row->service_name,
                    'plugin_name'=>$row->plugin_name,
                    'device_name'=>$row->device_name,
                    'device_id'=>$row->device_id  );
        }
        return $arr;
    }
    static function deleteCyclicMaintenance($dbconnection,$confid)
    {
        if(is_numeric($confid))
        {
            $query = $dbconnection->prepare("DELETE FROM esia_maintenance_configuration WHERE maintenance_conf_id=?");
            $query = parent::executeStatement($query,array($confid));
        }
        else
        {
            throw new EsiaDbException('$confid must be numeric');
        }

    }
    static function passNextMaintenance($dbconnection,$confid)
    {
        if(is_numeric($confid))
        {
            $query = $dbconnection->prepare("UPDATE esia_maintenance_configuration SET maintenance_conf_start_date=maintenance_conf_start_date +maintenance_conf_recurring WHERE maintenance_conf_id=?");
            $query = parent::executeStatement($query,array($confid));
        }
        else
        {
            throw new EsiaDbException('$confid must be numeric');
        }

    }
    static function addAdvanceNodeConfig($dbconnection,$nodeid,$config_type_id,$param)
    {
        $query = $dbconnection->prepare("INSERT INTO esia_device_advance_config(adv_id, adv_device_id, adv_config_type_id, adv_param)
                VALUES (DEFAULT,:nodeid,:configid,:param) RETURNING adv_id;");
        $query = parent::executeStatement($query,array(':nodeid'=>$nodeid,':configid'=>$config_type_id,':param'=>json_encode($param)));
        $row=$query->fetch(PDO::FETCH_OBJ);
        return $row->adv_id;
    }
    /**
     * @brief deleteAdvanceNodeConfig : Supprime la configuration avancée
     * @param PDO $dbconnection  l'objet de connexion PDO
     * @param integer $config_type_id identifiant DB de la configuratin avancé
     * @retval NONE
     * @exception une esiaDBException
     */
    static function deleteAdvanceNodeConfig($dbconnection,$config_type_id)
    {
        $query = $dbconnection->prepare("DELETE FROM esia_device_advance_config WHERE adv_id = ?");
        $query = parent::executeStatement($query,array($config_type_id));
    }
    /**
     * @brief updateAdvanceNodeConfig : met à jour les paramètres de configuration avancée
     * @param PDO $dbconnection  l'objet de connexion PDO
     * @param integer $id identifiant DB a mettre à jour
     * @param string| json les paramètres à envoyé
     * @retval integer l'id mis à jour
	 * @exception une esiaDBException
     */
    static function updateAdvanceNodeConfig($dbconnection,$id,$params)
    {
    	$query= $dbconnection->prepare("UPDATE esia_device_advance_config SET adv_param=:params
										WHERE adv_id = :id
										RETURNING adv_id;");

    	$query = parent::executeStatement($query,array(':id' =>$id,':params'=>$params));
    	$row=$query->fetch(PDO::FETCH_OBJ);
    	return $row->adv_id;
    }

	static function updateServiceProgramAlert($dbconnection,$alert_list)
	{
		if(is_array($alert_list))
		{
    		$arrids = array();
    		//si on charge un tableau associatif complets
			foreach($alert_list as $line)
    		{
    			if(!is_numeric($line['serviceid']))
    				throw new EsiaDbException('$node array must be numeric');
    			else
    				$arrids[]=$line['serviceid'];
    		}
    		$arrids = implode(',',$arrids);
    		$query = $dbconnection->prepare("DELETE FROM esia_service_response_program WHERE response_service_id IN ($arrids);");
    		$query = parent::executeStatement($query);
    		$query = $dbconnection->prepare("INSERT INTO esia_service_response_program (response_service_id, response_program_id, response_parameter,
            												response_ok, response_warning, response_critical, response_unknown)
    										VALUES (:serviceid,:program,:parameter,:ok,:warning,:critical,:unknown);");
			foreach($alert_list as $line)
    		{
    			if($line['progid'] != 0)
    			{
	    			$query = parent::executeStatement($query,array(':serviceid'=>$line['serviceid'],
	    					':program'=>$line['progid'],
	    					':parameter'=>$line['parameter'],
	    					':warning'=>$line['warning'],
	    					':critical'=>$line['critical'],
	    					':ok'=>$line['ok'],
	    					':unknown'=>$line['unknown']));
					Model_adm::reloadService($dbconnection,0,$line['serviceid']);
    			}
    		}
    	}
    }
    static function getGDPRRegister($dbconnection)
    {
    	$query = $dbconnection->prepare("SELECT register_id,register_user_name,to_char(register_date,'DD/MM/YYYY HH24:MI:SS') as register_date,register_action,register_type,register_data,register_finality FROM gdpr_register;");
    	$query = parent::executeStatement($query,array());
    	$arr = array();
    	while($row=$query->fetch(PDO::FETCH_OBJ))
    	{
    		$arr[] = array('id'=>$row->register_id,
    				'user'=>$row->register_user_name,
    				'date'=>$row->register_date,
    				'action'=>$row->register_action,
    				'type'=>$row->register_type,
    				'data'=>$row->register_data,
    				'finality'=>$row->register_finality);
    	}
    	return $arr;
    }
    static function addGDPRRegister($dbconnection,$action,$type,$data,$finality)
    {
    	$query = $dbconnection->prepare("INSERT INTO gdpr_register (register_id,register_user_name,register_date,register_action,register_type,register_data,register_finality) VALUES (DEFAULT,?,NOW(),?,?,?,?);");
    	global $user;
    	$query = parent::executeStatement($query,array($user->getName(),$action,$type,$data,$finality ));

    }
	public static function updateAuthFile($dbconnection,$adv_id,$type,$data,$node_id=null)
    {
		if(empty($node_id))
        {
            $setting = Model_adm::getAdvanceNodeConfig($dbconnection,NULL,$adv_config_id)[0];
            $node_id= $setting['node_id'];
        }
		$file_path = $type.'_'.$node_id;
		$sql = 'UPDATE esia_device_advance_config SET adv_authfile_crypt = :file_path WHERE adv_id = :id';

		if( $data !== FALSE)
        {
            insertSettings($dbconnection,$adv_id,$file_path,$data,true);
        }

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query,array(
				':id' => $adv_id,
				':file_path' => $file_path
		));
	}

	public static function removeAuthFile($dbconnection,$adv_id){
		removeSettings($dbconnection,$adv_id);
	}

	/**
			@brief Crée un nouveau noeud qui est une copie du précédent
			@details
				Fait une copie intégrale du noeud $src_node en créant un tout nouveau noeud.
			@param Device|integer $src_node Le noeud source sous forme d'un device id ou d'un objet Device
			@param NULL|STRING $new_name Le nom de la copie, si NULL ce sera le nom de $src_node précédé de 'COPY_'
			@param NULL|STRING $new_addr Remplace l'address IP de $src_node par une nouvelle
			@param boolean $advance_config Copie aussi les configuration avancé et les fichiés authfile
			@param boolean $node_group_access Copie aussi les accés par les groupes de noeuds au device
			@param boolean $services Copie aussi les services et les pattern
			@param boolean alert Copie aussi les alertes configurées pour les services
	*/
	public static function copyDeviceNode($dbconnection,$src_node,$new_name,$new_addr,$advance_config=TRUE,$node_group_access=TRUE,$services=TRUE,$alert=TRUE)
	{
		if( !$src_node instanceof Device)
		{
			$src_node = Model_adm::InstanceNode($dbconnection,$src_node);
		}
		$new_node = $src_node;
		if( empty($new_name) )
		{
			$new_name = 'COPY_'. $src_node->getName();
		}
		if( empty($new_addr) )
		{
			$new_addr = $src_node->getIPAddress();
		}

		$new_node->setID(NULL);
		$new_node->setName($new_name);
		$new_node->setIPAddress($new_addr);

		$src_node_id = $src_node->getID();
		$new_node_id = Model_adm::addNode($dbconnection,$new_node);
		$new_node->setID($new_node_id);

		if( $advance_config )
		{
			Model_adm::copyNodeAdvanceConfig($dbconnection,$src_node_id,$new_node_id);
		}
		if( $node_group_access )
		{
			Model_adm::copyNodeUserGroupAccess($dbconnection,$src_node_id,$new_node_id,TRUE);
		}
		else
		{
			Model_adm::addDeviceInDefaultGroup($dbconnection,$new_node);
		}
		if( $services )
		{
			Model_adm::copyNodeServicesAndPatterns($dbconnection,$src_node_id,$new_node_id,$alert);
		}
		return $new_node_id;
	}

	/**
			@brief Copie les configurations advance_config d'un noeud vers un autre
			@details
				Fait une copie intégrale de tous les advance node config d'un noeud $src_node
				vers un autre noeud $dst_node. Copie aussi les fichiers d'authentication chiffrés
			@param Device|integer $src_node Le noeud source sous forme d'un device id ou d'un objet Device
			@param Device|integer $dst_node Le noeud destination sous forme d'un device id ou d'un objet Device
			@param boolean $force FALSE par défaut, si FALSE, ne copie pas et lance une exception si le noeud destination
				posséde déjà 1 ou plusieurs advance_config
	*/
	public static function copyNodeAdvanceConfig($dbconnection,$src_node,$dst_node,$force=FALSE)
	{
		//Préparation des paramètres pour la requête
		//REM : $dst_node est concaténé directement dans la request, mais il est de type integer
		if( $src_node instanceof Device)
		{
			$src_node = $src_node->getID();
		}
		if( $dst_node instanceof Device)
		{
			$dst_node = $dst_node->getID();
		}

		if( $force == FALSE && !empty(Model_adm::getAdvanceNodeConfig($dbconnection,$dst_node)) )
		{
			throw new Exception('Destination already have several advance_node settings');
		}

		$params[':src_node_id_str'] = "$src_node";
		$params[':dst_node_id_str'] = "$dst_node";

		$src_node = intval($src_node);
		$dst_node = intval($dst_node);

		$params[':src_node_id_int'] = $src_node;

		//Création et exécution de la requête
		$sql = '';
		$sql.="INSERT INTO esia_device_advance_config(adv_device_id,adv_config_type_id,adv_param,adv_authfile_crypt,adv_plugin_keyword_value)\n";
		$sql.="SELECT '$dst_node' AS adv_device_id,adv_config_type_id,adv_param,replace(adv_authfile_crypt,:src_node_id_str,:dst_node_id_str) AS adv_authfile_crypt,replace(adv_plugin_keyword_value,:src_node_id_str,:dst_node_id_str) AS adv_plugin_keyword_value\n";
		$sql.="FROM esia_device_advance_config\n";
		$sql.="WHERE adv_device_id=:src_node_id_int\n";
		$sql.="RETURNING adv_authfile_crypt AS dst_path;";

		$query = $dbconnection->prepare($sql);
		$query = Model_adm::executeStatement($query,$params);

		//Copy des fichiers d'authentication
		$decrypt_dir = FALSE;
		foreach($query->fetchAll(PDO::FETCH_ASSOC) as $row)
		{
			$dst_name_file = $row['dst_path'];
			$src_name_File = str_replace($dst_node,$src_node,$dst_name_file);

			//Copy du fichier chiffré
			$src_name_File_path = CIPHER_FILES_DIR."/$src_name_File.auth";
			$dst_name_file_path = CIPHER_FILES_DIR."/$dst_name_file.auth";

			if (!copy($src_name_File_path, $dst_name_file_path)) {
    			throw new Exception('Failed to copy advance config authfile');
			}

			//Copy du fichier non chiffré
			$src_name_File_path = CLEAR_FILES_DIR."/$src_name_File.dat";
			$dst_name_file_path = CLEAR_FILES_DIR."/$dst_name_file.dat";
			if (!copy($src_name_File_path, $dst_name_file_path)) {
					$decrypt_dir = TRUE;
			}
		}

		//Si la src d'un des fichier non-chiffré n'existait pas, alors on décrypte tous les fichiers chiffrés.
		if( $decrypt_dir )
		{
			exec('php /usr/local/esia/keystore/scripts/prepare_auth_files.php');
		}
	}

	/**
			@brief Copie les accès groupes utilisateurs d'un noeud vers un autre
			@details
				Fait une copie intégrale de tous les les accès groupes utilisateurs d'un noeud $src_node
				vers un autre noeud $dst_node.
			@param Device|integer $src_node Le noeud source sous forme d'un device id ou d'un objet Device
			@param Device|integer $dst_node Le noeud destination sous forme d'un device id ou d'un objet Device
			@param boolean $force FALSE par défaut, si FALSE, ne copie pas et lance une exception si le noeud destination
				posséde déjà certains des accès du noeud précédent
	*/
	public static function copyNodeUserGroupAccess($dbconnection,$src_node,$dst_node,$force=FALSE)
	{
		//Préparation des paramètres pour la requête
		//REM : $dst_node est concaténé directement dans la request, mais il est de type integer
		if( $src_node instanceof Device)
		{
			$src_node = $src_node->getID();
		}
		if( $dst_node instanceof Device)
		{
			$dst_node = $dst_node->getID();
		}

		$src_node = intval($src_node);
		$dst_node = intval($dst_node);

		$params[':src_node_id_int'] = $src_node;

		//Création et exécution de la requête
		$sql = '';
		$sql.= "INSERT INTO esia_group_device_access(access_device_id,access_group_id)\n";
		$sql.= "SELECT '$dst_node' AS access_device_id,access_group_id FROM esia_group_device_access\n";
		$sql.= "WHERE access_device_id = :src_node_id_int\n";
		if( $force )
		{
			$sql.= "ON CONFLICT DO NOTHING";
		}

		$query = $dbconnection->prepare($sql);
		$query = Model_adm::executeStatement($query,$params);
	}

	/**
			@brief Copie les services et patterns d'un noeud vers un autre
			@details
				Fait une copie intégrale de tous services et patterns d'un noeud $src_node
				vers un autre noeud $dst_node. Si le noeud destination posséde déjà un service de même nom,
				alors ce service n'est pas recopié. Les programmes d'alertes sont aussi copiés.
			@param Device|integer $src_node Le noeud source sous forme d'un device id ou d'un objet Device
			@param Device|integer $dst_node Le noeud destination sous forme d'un device id ou d'un objet Device
			@param boolean $alert_program TRUE par défaut. Mettre à false pour désactiver la copie des programs d'alerte
	*/
	public static function copyNodeServicesAndPatterns($dbconnection,$src_node,$dst_node,$alert_program = TRUE)
	{
		//Préparation des paramètres pour la requête
		//REM : $dst_node est concaténé directement dans la request, mais il est de type integer
		if( $src_node instanceof Device)
		{
			$src_node = $src_node->getID();
		}
		if( $dst_node instanceof Device)
		{
			$dst_node = $dst_node->getID();
		}

		$src_node = intval($src_node);
		$dst_node = intval($dst_node);

		$params[':src_node_id_int'] = $src_node;
		$params[':dst_node_id_int'] = $dst_node;

		$sql = '';
		$sql.= "SELECT service_device_id,service_plugin_id,service_interval_time,service_parameter,service_name,service_included_id,service_priority,response_program_id,response_parameter,response_ok,response_warning,response_critical,response_unknown\n";
		$sql.= "FROM esia_services LEFT OUTER JOIN esia_service_response_program\n";
		$sql.= "ON service_id = response_service_id\n";
		$sql.= "WHERE service_device_id = :src_node_id_int AND service_name NOT IN(SELECT service_name FROM esia_services WHERE service_device_id=:dst_node_id_int)";

		$query = $dbconnection->prepare($sql);
		$query = Model_adm::executeStatement($query,$params);

		$sql_services_insert = '';
		$sql_services_insert .= "INSERT INTO esia_services(service_previous_state,service_device_id,service_plugin_id,service_interval_time,service_parameter,service_name,service_included_id,service_priority)\n";
		$sql_services_insert .= "VALUES(:service_previous_state,:service_device_id,:service_plugin_id,:service_interval_time,:service_parameter,:service_name,:service_included_id,:service_priority) RETURNING service_id";

		$response_params_list = array();
		foreach($query->fetchAll(PDO::FETCH_ASSOC) as $row)
		{
			$services_params = array();
			$response_params = array();
			$response_params[':response_service_id'] = NULL;
			foreach($row as $key => $value)
			{
				$key_words = explode('_',$key);
				if( $key_words[0] == 'service' )
				{
					$services_params[":$key"] = $value;
				}
				else
				{
					$response_params[":$key"] = $value;
				}
			}

			$services_params[':service_device_id'] = $dst_node;
			$services_params[':service_previous_state'] = 4;
			$query = $dbconnection->prepare($sql_services_insert);
			$query = Model_adm::executeStatement($query,$services_params);

			model_adm::addGDPRRegister($dbconnection,'ADD','SERVICE',$lang['GDPR_ADD_SERVICE']. $services_params[':service_name'],$lang['GDPR_ADD_SERVICE_FINALITY']);

			$service_id = $query->fetchAll(PDO::FETCH_ASSOC);
			$service_id = array_shift($service_id);
			$service_id = $service_id['service_id'];

			$query= $dbconnection->prepare("INSERT INTO esia_service_pending_execute(pending_service_id)VALUES (:id);");
			$query = parent::executeStatement($query,array(':id'=>$service_id));

			if(!empty($response_params[':response_program_id']) && $alert_program)
			{
				$response_params[':response_service_id'] = $service_id;
				$response_params_list[] = array(
					'serviceid' => $service_id,
					'progid' => $response_params[':response_program_id'],
					'parameter' => $response_params[':response_parameter'],
					'warning' => intval($response_params[':response_warning']),
					'critical' => intval($response_params[':response_critical']),
					'ok' => intval($response_params[':response_ok']),
					'unknown' => intval($response_params[':response_unknown']),
				);
			}
			else
			{   //Les services de ce else doivent être reloader.
				//Les autres sont reloader dans la fonction updateServiceProgramAlert ci-dessous
				Model_adm::reloadService($dbconnection,0,$service_id);
			}
		}
		if(!empty($response_params_list))
		{
			model_adm::updateServiceProgramAlert($dbconnection,$response_params_list);
		}
	}
	/**
	 * @brief Get pattern where a plugin is included
	 * @param PDO $dbconnection
	 * @param integer $plugin id of the plugin included in the patterns
	 * @param integer $service id of the service included by the pattern
	 * @retval array list of patterns
	 * @exception esiaDBException
	 */
	public static function getPatternIncluded($dbconnection, $plugin=NULL, $service=NULL)
	{
		$sql = "SELECT pattern_id, pattern_name, pattern_description, included_id, included_parameter
			FROM esia_service_patterns
			LEFT JOIN esia_service_pattern_included ON esia_service_pattern_included.included_pattern_id = esia_service_patterns.pattern_id";

		$params = array();
		if(!empty($plugin) && is_numeric($plugin))
		{
			$sql .= " WHERE included_plugin_id = :plugin";
			$params[':plugin'] = $plugin;
		}
		elseif(!empty($service) && is_numeric($service))
		{
			$sql .= " WHERE included_id = :service";
			$params[':service'] = $service;
		}
		$query = $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query, $params);

		$patterns = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
        {
            $patterns[$row->included_id] = array(
                'id' => $row->pattern_id,
                'name' => $row->pattern_name,
                'description' => $row->pattern_description,
				'parameter' => $row->included_parameter,
				'included_id' => $row->included_id,
            );
        }

		return $patterns;
	}
	/**
	 * @brief Get rrd info by plugin id
	 * @param PDO $dbconnection
	 * @param integer (optional) $plugin id of the plugin to fetch the RRDs. If empty get all RRDs for all plugins
	 * @retval array list of RRDs
	 * @exception esiaDBException
	 */
	public static function getRRDByPlugin($dbconnection, $plugin=NULL)
	{
		$sql = "
			SELECT rrd_plugin_id, rrd_type, rrd_regex, rrd_time, rrd_id, rrd_name, rrd_min, rrd_max, rrd_vertical_label, rrd_graph_type, rrd_graph_color, rrd_regex_threshold, COALESCE(rrd_graph_num, 0) AS graph_num_not_null, rrd_calc
			FROM rrd_plugins
		";

		$params = array();
		if(!empty($plugin) && is_numeric($plugin))
		{
			$sql .= " WHERE rrd_plugin_id = :plugin";
			$params[':plugin'] = $plugin;
		}

		$sql .= " ORDER BY rrd_plugin_id,graph_num_not_null,rrd_name;";

		$query = $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query, $params);

		$rrdList = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			$rrdList[$row->rrd_plugin_id][$row->rrd_id] = array(
				'id' => $row->rrd_id,
				'name' => $row->rrd_name,
				'unit' => $row->rrd_vertical_label,
				'calc' => $row->rrd_calc,
				'type' => $row->rrd_graph_type,
				'c3_type' => Model_esia::RRD_GRAPH_TYPES_TO_C3[$row->rrd_graph_type],
				'regex' => $row->rrd_regex,
				'color' => $row->rrd_graph_color,
				'num' => $row->graph_num_not_null,
				'min' => $row->rrd_min,
				'max' => $row->rrd_max,
				'rrd_type' => $row->rrd_type,
				'time' => $row->rrd_time,
				'threshold' => $row->rrd_regex_threshold,
			);
		}

		return $rrdList;
	}
	/**
	 * @brief Add rrd info by plugin id
	 * @param PDO $dbconnection
	 * @param integer $pluginId id of the plugin to update the RRDs
	 * @param array $rrdData data to update
	 * @exception esiaDBException
	 */
	public static function addRRDPlugin($dbconnection, $pluginId, $rrdData)
	{
		if(empty($pluginId) || !is_numeric($pluginId))
		{
			throw new EsiaDbException("Plugin ID is not numeric");
		}

		// Check data format
		$validTypes = array_keys(Model_esia::RRD_GRAPH_TYPES_TO_C3);
		if(!isset($rrdData['type']) || !in_array($rrdData['type'], $validTypes))
		{
			throw new EsiaDbException("RRD type is invalid. Expected: " . implode(', ', $validTypes));
		}
		if(!isset($rrdData['color']) || !preg_match('/^#[a-f0-9]{6}$/i', $rrdData['color']))
		{
			throw new EsiaDbException("RRD color is invalid. Expected hexadecimal color (#000000)");
		}
		else
		{
			$rrdData['color'] = strtoupper($rrdData['color']);
		}

		$params = array(
			':plugin' => $pluginId,
			':type' => $rrdData['rrd_type'],
			':regex' => $rrdData['regex'],
			':time' => $rrdData['time'],
			':name' => $rrdData['name'],
			':min' => $rrdData['min'],
			':max' => $rrdData['max'],
			':vertical_label' => $rrdData['unit'],
			':graph_type' => $rrdData['type'],
			':graph_color' => $rrdData['color'],
			':regex_threshold' => $rrdData['threshold'],
			':calc' => $rrdData['calc'],
		);

		// If num = 0 set as NULL
		$sqlNumVal = 'NULL';
		if(!empty($rrdData['num']))
		{
			$sqlNumVal = ':graph_num';
			$params[':graph_num'] = $rrdData['num'];
		}

		$sql = "INSERT INTO rrd_plugins(
			rrd_plugin_id, rrd_type, rrd_regex, rrd_time, rrd_name, rrd_min, rrd_max, rrd_vertical_label, rrd_graph_type, rrd_graph_color, rrd_regex_threshold, rrd_graph_num, rrd_calc)
			VALUES (:plugin, :type, :regex, :time, :name, :min, :max, :vertical_label, :graph_type, :graph_color, :regex_threshold, $sqlNumVal, :calc)";

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);
	}
	/**
	 * @brief Update rrd info by plugin id
	 * @param PDO $dbconnection
	 * @param integer $pluginId id of the plugin to update the RRDs
	 * @param integer $rrdId id of the rrd to update
	 * @param array $rrdData data to update
	 * @exception esiaDBException
	 */
	public static function updateRRDPlugin($dbconnection, $pluginId, $rrdId, $rrdData)
	{
		if(empty($pluginId) || !is_numeric($pluginId))
		{
			throw new EsiaDbException("Plugin ID is not numeric");
		}
		if(empty($rrdId) || !is_numeric($rrdId))
		{
			throw new EsiaDbException("RRD ID is not numeric");
		}

		// Check data format
		$validTypes = array_keys(Model_esia::RRD_GRAPH_TYPES_TO_C3);
		if(!isset($rrdData['type']) || !in_array($rrdData['type'], $validTypes))
		{
			throw new EsiaDbException("RRD type is invalid. Expected: " . implode(', ', $validTypes));
		}
		if(!isset($rrdData['color']) || !preg_match('/^#[a-f0-9]{6}$/i', $rrdData['color']))
		{
			throw new EsiaDbException("RRD color is invalid. Expected hexadecimal color (#000000)");
		}
		else
		{
			$rrdData['color'] = strtoupper($rrdData['color']);
		}

		$sql = "UPDATE rrd_plugins SET rrd_graph_type = :type, rrd_graph_color = :color
			WHERE rrd_plugin_id = :plugin
			AND rrd_id = :id
			RETURNING rrd_id";

		$params = array(
			':id' => $rrdId,
			':plugin' => $pluginId,
			':type' => $rrdData['type'],
			':color' => $rrdData['color'],
		);

		$query = $dbconnection->prepare($sql);
		$query = Model_esia::executeStatement($query, $params);

		if($query->rowCount() <= 0)
		{
			throw new EsiaDbException('RRD does not exist');
		}
	}
	/**
	 * @brief Update plugin default alert program
	 * @param PDO $dbconnection
	 * @param integer $pluginId id of the plugin to update the alert program
	 * @param array $alertData data to update
	 * @exception esiaDBException
	 */
	public static function updatePluginAlertProgram($dbconnection, $pluginId, $alertData)
	{
		if(empty($pluginId) || !is_numeric($pluginId))
		{
			throw new EsiaDbException("Plugin ID is not numeric");
		}
		if(!is_array($alertData))
		{
			throw new EsiaDbException("Alert data must be an array");
		}
		if(!isset($alertData['alerting_software']) || !is_numeric($alertData['alerting_software']))
		{
			throw new EsiaDbException("Alert program ID must be numeric");
		}

		$params = array(':plugin_id' => $pluginId);
		// Remove previous configuration
		$query = $dbconnection->prepare("DELETE FROM esia_plugin_react_program WHERE react_plugin_id = :plugin_id;");
		$query = Model_esia::executeStatement($query, $params);

		// If program is set add the new one (0 can be send to only delete program)
		if($alertData['alerting_software'] > 0)
		{
			$sql = "INSERT INTO esia_plugin_react_program(
				react_plugin_id, react_program_id, react_parameter, react_ok, react_warning, react_critical, react_unknown)
				VALUES (:plugin_id, :program_id, :parameter, :ok, :warning, :critical, :unknown);";
	
			$params[':program_id'] = $alertData['alerting_software'];
			$params[':parameter'] = $alertData['alerting_parameter'];
			$params[':ok'] = empty($alertData['ok']) ? 0 : 1;
			$params[':warning'] = empty($alertData['warning']) ? 0 : 1;
			$params[':critical'] = empty($alertData['critical']) ? 0 : 1;
			$params[':unknown'] = empty($alertData['unknown']) ? 0 : 1;
	
			$query = $dbconnection->prepare($sql);
			$query = Model_esia::executeStatement($query, $params);
		}
	}
	/**
	 * @brief ajoute un canal RSS
	 * @param PDO $dbconnection
	 * @param string $name nom du canal
	 * @param string $description description du canal
	 * @param string $type type du canal (LOCAL ou REMOTE)
	 * @param string $url url de connexion au channel
	 * @param string $lang langue du canal
	 * @param string $image path vers le fichier image
 	* @retval integer id du nouveau canal
	 * @exception esiaDBException
	 */
	public static function addRSSChannel($dbconnection, $name, $description, $type, $url,$lang,$image,$enable,$rss_category,$usergroup=NULL)
	{
		$query = $dbconnection->prepare("INSERT INTO public.rss_channel(
			channel_name, channel_description, channel_language, channel_image, channel_type, channel_url,channel_enable)
			VALUES (:name,:descr,:lang,:image,:type,:url,:enable) RETURNING channel_id;");
		$param[':name']=$name;
		$param[':descr']=$description;
		$param[':lang']=$lang;
		$param[':image']=$image;
		$param[':type']=$type;
		$param[':url']=$url;
		$param[':enable']=$enable;
		$query = Model_esia::executeStatement($query, $param);
		$row=$query->fetch(PDO::FETCH_OBJ);
		
		if(!empty($rss_category))
		{
			$querycate = $dbconnection->prepare("INSERT INTO public.rss_link_channel_to_category(link_channel_id, link_category_id)
				VALUES (?,?);");
			foreach($rss_category as $cat)
			{
				$querycate = Model_esia::executeStatement($querycate, array($row->channel_id,$cat));
			}
		}
		if(!empty($usergroup))
		{
			$group = implode(',',$usergroup);	
			$query = $dbconnection->prepare("UPDATE rss_channel SET channel_group_link=:group WHERE channel_id = :id");
			$query = Model_esia::executeStatement($query, array('id'=> $row->channel_id,':group'=>$group));
		}
		return $row->channel_id;
	}
	/**
	 * @brief met à jour un canal RSS
	 * @param PDO $dbconnection
	 * @param integer $id identifiant du canal
	 * @param string $name nom du canal
	 * @param string $description description du canal
	 * @param string $type type du canal (LOCAL ou REMOTE)
	 * @param string $url url de connexion au channel
	 * @param string $lang langue du canal
	 * @param string $image path vers le fichier image
	 * @retval integer id du nouveau canal
	 * @exception esiaDBException
	 */
	public static function updateRSSChannel($dbconnection,$id, $name, $description, $type, $url,$lang,$image,$enable,$rss_category,$usergroup=NULL)
	{
		$query = $dbconnection->prepare("UPDATE rss_channel SET channel_name=:name, channel_description = :descr, channel_language=:lang,
							channel_type=:type, channel_url=:url, channel_image=:image, channel_enable=:enable
							WHERE channel_id = :id");
		$param[':id']=$id;
		$param[':name']=$name;
		$param[':descr']=$description;
		$param[':lang']=$lang;
		$param[':image']=$image;
		$param[':type']=$type;
		$param[':url']=$url;
		$param[':enable']=$enable;
		$query = Model_esia::executeStatement($query, $param);
		
		if(!empty($rss_category))
		{
			$querycate = $dbconnection->prepare("DELETE FROM rss_link_channel_to_category WHERE link_channel_id=?");
			$querycate = Model_esia::executeStatement($querycate, array($id));
			$querycate = $dbconnection->prepare("INSERT INTO rss_link_channel_to_category(link_channel_id, link_category_id)
				VALUES (?,?);");
			foreach($rss_category as $cat)
			{
				$querycate = Model_esia::executeStatement($querycate, array($id,$cat));
			}
		}
		if(!empty($usergroup))
		{
			$group = implode(',',$usergroup);
			$query = $dbconnection->prepare("UPDATE rss_channel SET channel_group_link=:group WHERE channel_id = :id");
			$query = Model_esia::executeStatement($query, array('id'=> $id,':group'=>$group));
		}
		return $id;
	}
	/**
	 * @brief Supprime un canal RSS
	 * @param PDO $dbconnection
	 * @param integer $id identifiant du canal
	 * @retval bool true si tous c'est bien passé
	 * @exception esiaDBException
	 */
	public static function deleteRSSChannel($dbconnection,$id)
	{
		$rssChan = Model_esia::getRSSChannel($dbconnection,$id);
		unlink($rssChan[0]['image']);
		$query = $dbconnection->prepare("DELETE FROM rss_link_channel_to_category WHERE link_channel_id = ?");
		$query = Model_esia::executeStatement($query, array($id));
		$query = $dbconnection->prepare("DELETE FROM rss_user_subscribe WHERE subscribe_channel_id=  ?");
		$query = Model_esia::executeStatement($query, array($id));
		$query = $dbconnection->prepare("DELETE FROM rss_user_read WHERE read_message_id IN (SELECT message_id FROM rss_message WHERE message_channel_id = ?)");
		$query = Model_esia::executeStatement($query, array($id));
		$query = $dbconnection->prepare("DELETE FROM rss_message WHERE message_channel_id = ?");
		$query = Model_esia::executeStatement($query, array($id));
		$query = $dbconnection->prepare("DELETE FROM rss_channel WHERE channel_id = ?");
		$query = Model_esia::executeStatement($query, array($id));
		return true;
	}
	/**
	 * @brief ajoute un canal RSS
	 * @param PDO $dbconnection
	 * @param string $name nom du canal
	 * @param string $description description du canal
	 * @param string $type type du canal (LOCAL ou REMOTE)
	 * @param string $url url de connexion au channel
	 * @param string $lang langue du canal
	 * @param string $image path vers le fichier image
	 * @retval integer id du nouveau canal
	 * @exception esiaDBException
	 */
	public static function addRSSCategory($dbconnection, $name, $description)
	{
		$query = $dbconnection->prepare("INSERT INTO rss_category(category_name, category_description)
								VALUES (:name,:descr) RETURNING category_id;");
		$param[':name']=$name;
		$param[':descr']=$description;
		$query = Model_esia::executeStatement($query, $param);
		$row=$query->fetch(PDO::FETCH_OBJ);
		return $row->category_id;
	}
	/**
	 * @brief met à jour un canal RSS
	 * @param PDO $dbconnection
	 * @param integer $id identifiant du canal
	 * @param string $name nom du canal
	 * @param string $description description du canal
	 * @param string $type type du canal (LOCAL ou REMOTE)
	 * @param string $url url de connexion au channel
	 * @param string $lang langue du canal
	 * @param string $image path vers le fichier image
	 * @retval integer id du nouveau canal
	 * @exception esiaDBException
	 */
	public static function updateRSSCategory($dbconnection,$id, $name, $description)
	{
		$query = $dbconnection->prepare("UPDATE rss_category SET category_name=:name, category_description = :descr WHERE category_id = :id");
		$param[':id']=$id;
		$param[':name']=$name;
		$param[':descr']=$description;
		$query = Model_esia::executeStatement($query, $param);
		$row=$query->fetch(PDO::FETCH_OBJ);
		return $row->category_id;
	}
	/**
	 * @brief Supprime une categorie RSS
	 * @param PDO $dbconnection
	 * @param integer $id identifiant de la categorie
	 * @retval bool true si tous c'est bien passé
	 * @exception esiaDBException
	 */
	public static function deleteRSSCategory($dbconnection,$id)
	{
		$query = $dbconnection->prepare("DELETE FROM rss_link_channel_to_category WHERE link_category_id = ?");
		$query = Model_esia::executeStatement($query, array($id));
		$query = $dbconnection->prepare("DELETE FROM rss_category WHERE category_id = ?");
		$query = Model_esia::executeStatement($query, array($id));
		return true;
	}
	/**
	 * @brief Ajoute un message à un flux
	 * @param PDO $dbconnection
	 * @param integer $id identifiant de la categorie
	 * @retval bool true si tous c'est bien passé
	 * @exception esiaDBException
	 */
	public static function addMessage($dbconnection,$channel,$title,$message,$link,$author,$urlimage,$rsscategory,$sign=null,$date=null)
	{
		$sql = "INSERT INTO rss_message(message_id,message_date_publish,message_channel_id,
					message_title, message_description,message_link, message_author, message_image,	message_categories)
					VALUES (DEFAULT,NOW(),:chan,:title,:message,:url,:author,:image,:cate) RETURNING message_id;";
		if(!empty($sign))
		{
			$sql = "INSERT INTO rss_message(message_title, message_description, message_link, message_author,
								 message_image, message_channel_id,message_date_publish, message_sign,message_categories)
	              SELECT :title,:message,:url,:author,:image,:chan,:datepublish,:sign,:cate
                  WHERE NOT EXISTS( SELECT message_id FROM rss_message WHERE message_sign=:checksign)
					RETURNING message_id;";
			$param[':sign']=$sign;
			$param[':checksign']=$sign;
			$param[':datepublish']=$date;
		}
		$query = $dbconnection->prepare($sql);
		$param[':chan']=$channel;
		$param[':title']=$title;
		$msg = htmlentities($message);	
		$param[':message']=mb_convert_encoding($msg, "UTF-8");
		$param[':url']=$link;
		$param[':author']=$author;
		$param[':image']=$urlimage;
		$param[':cate']=$rsscategory;
		$query = Model_esia::executeStatement($query, $param);
		$msgid=null;
		if($query->rowCount() > 0)
		{
			$row=$query->fetch(PDO::FETCH_OBJ);
			$msgid = $row->message_id;
		}
		if(!empty($sign))
		{
			$queryup = $dbconnection->prepare("UPDATE rss_channel SET channel_last_publication = NOW() WHERE channel_id =?");
			$queryup = Model_esia::executeStatement($queryup, array($channel));
		}
		return $msgid;
	}
	public static function deleteRSSMessage($dbconnection,$id)
	{
		$param=array();
		$sqluser = "DELETE FROM rss_user_read";
		$sqlmessage="DELETE FROM rss_message";
		if(is_array($id))
		{
			$insql = model_esia::pdo_array_to_SQL('msgid', $id , $param);
			$sqluser .= " WHERE read_message_id IN ($insql)";
			$sqlmessage .= " WHERE message_id IN ($insql)";
		}
		else
		{
			$sqluser .= " WHERE read_message_id = :msgid";
			$sqlmessage .= " WHERE message_id = :msgid";
			$param['msgid'] = $id;
		}
		$query = $dbconnection->prepare($sqluser);
		$query = Model_esia::executeStatement($query, $param);
		
		$query = $dbconnection->prepare($sqlmessage);
		$query = Model_esia::executeStatement($query, $param);
	}
	public static function purgeRSSOldMessage($dbconnection,$day,$channel=null)
	{
		$sql = "SELECT message_id
				FROM rss_message
				WHERE message_date_publish < NOW() - cast(:day as interval)";
		$param[':day']=$day.' day';
		if(is_numeric($channel))
		{
			$sql .= " AND message_channel_id=:chan";
			$param[':chan']=$channel;
		}
		$query = $dbconnection->prepare("DELETE FROM rss_user_read WHERE read_message_id IN ($sql)");
		$query = Model_esia::executeStatement($query, $param);
		
		$query = $dbconnection->prepare("DELETE FROM rss_message WHERE message_id IN ($sql)");
		$query = Model_esia::executeStatement($query, $param);
	}
}
?>
